@@ -31,10 +31,10 @@
 #include "cuda-autostep.h"
 #include "cuda-coord-set.h"
 #include "cuda-frame.h"
+#include "cuda-modules.h"
 #include "cuda-options.h"
 #include "cuda-state.h"
 #include "cuda-tdep.h"
-#include "cuda-modules.h"
 
 /* When inside an autostep range, we go into single-step mode.
    When true, it means we are actively handling an autostep region.  Otherwise
@@ -78,15 +78,14 @@ autostep_report_exception_host (uint64_t before_pc)
 
   struct symtab_and_line before_sal = find_pc_line (before_pc, 0);
 
-  gdb_printf (
-      _ ("Autostep precisely caught exception at %s:%d (0x%llx)\n"),
-      before_sal.symtab->filename, before_sal.line,
-      (unsigned long long)before_pc);
+  gdb_printf (_ ("Autostep precisely caught exception at %s:%d (0x%llx)\n"),
+	      before_sal.symtab->filename, before_sal.line,
+	      (unsigned long long)before_pc);
 
   set_internalvar (lookup_internalvar ("autostep_exception_pc"),
-                   value_from_longest (type_data_ptr, (LONGEST)before_pc));
+		   value_from_longest (type_data_ptr, (LONGEST)before_pc));
   set_internalvar (lookup_internalvar ("autostep_exception_line"),
-                   value_from_longest (type_uint32, (LONGEST)before_sal.line));
+		   value_from_longest (type_uint32, (LONGEST)before_sal.line));
 }
 
 /* Report an exception in device code based on the number of steps NSTEPS, the
@@ -95,7 +94,7 @@ autostep_report_exception_host (uint64_t before_pc)
 
 static void
 autostep_report_exception_device (int nsteps, int before_ln,
-                                  uint64_t before_pc, uint64_t after_pc)
+				  uint64_t before_pc, uint64_t after_pc)
 {
   struct gdbarch *gdbarch = get_current_arch ();
   struct type *type_uint32 = builtin_type (gdbarch)->builtin_uint32;
@@ -110,7 +109,8 @@ autostep_report_exception_device (int nsteps, int before_ln,
   /* If the thread before stepping is also active, the exception didn't occur
      in a divergent thread */
   const auto &p = cuda_current_focus::get ().physical ();
-  const bool divergent = !cuda_state::lane_divergent (p.dev (), p.sm (), p.wp (), before_ln);
+  const bool divergent
+      = !cuda_state::lane_divergent (p.dev (), p.sm (), p.wp (), before_ln);
 
   /* Calculate exception PC - if more than 1 one instruction was executed
    * it means there was no control flow instructions so after_pc should
@@ -120,16 +120,16 @@ autostep_report_exception_device (int nsteps, int before_ln,
    * instruction. */
   if (nsteps > 1)
     cuda_debugapi::get_adjusted_code_address (
-        p.dev (), after_pc, &exception_pc, CUDBG_ADJ_PREVIOUS_ADDRESS);
+	p.dev (), after_pc, &exception_pc, CUDBG_ADJ_PREVIOUS_ADDRESS);
   else
     exception_pc = cuda_sstep_get_last_pc ();
 
   cuda_trace_domain (CUDA_TRACE_BREAKPOINT,
-                     "Autostep: nsteps %d divergent %d after_pc 0x%llx "
-                     "exception_pc 0x%llx last_pc 0x%llx",
-                     nsteps, divergent, (long long)after_pc,
-                     (long long)exception_pc,
-                     (long long)cuda_sstep_get_last_pc ());
+		     "Autostep: nsteps %d divergent %d after_pc 0x%llx "
+		     "exception_pc 0x%llx last_pc 0x%llx",
+		     nsteps, divergent, (long long)after_pc,
+		     (long long)exception_pc,
+		     (long long)cuda_sstep_get_last_pc ());
 
   /* We could have stepped a control flow instruction like NOP.S or SYNC, which
    * means another set of lanes ran while we tried to step the lowest active
@@ -138,14 +138,14 @@ autostep_report_exception_device (int nsteps, int before_ln,
   if (!cuda_sstep_lane_stepped (before_ln))
     {
       cuda_coords filter{
-        p.dev (),          p.sm (),
-        p.wp (),           cuda_sstep_get_lowest_lane_stepped (),
-        CUDA_WILDCARD,     CUDA_WILDCARD,
-        CUDA_WILDCARD_DIM, CUDA_WILDCARD_DIM,
-        CUDA_WILDCARD_DIM
+	p.dev (),	   p.sm (),
+	p.wp (),	   cuda_sstep_get_lowest_lane_stepped (),
+	CUDA_WILDCARD,	   CUDA_WILDCARD,
+	CUDA_WILDCARD_DIM, CUDA_WILDCARD_DIM,
+	CUDA_WILDCARD_DIM, CUDA_WILDCARD_DIM
       };
       cuda_coord_set<cuda_coord_set_type::threads, select_valid | select_sngl>
-          lane{ filter };
+	  lane{ filter };
       gdb_assert (lane.size ());
       switch_to_cuda_thread (*lane.begin ());
     }
@@ -156,23 +156,22 @@ autostep_report_exception_device (int nsteps, int before_ln,
 
   if (exception_sal.symtab && exception_sal.line)
     snprintf (exception_pc_line_info, sizeof (exception_pc_line_info),
-              "%s:%d (0x%llx)", exception_sal.symtab->filename,
-              exception_sal.line, (unsigned long long)exception_pc);
+	      "%s:%d (0x%llx)", exception_sal.symtab->filename,
+	      exception_sal.line, (unsigned long long)exception_pc);
   else
     snprintf (exception_pc_line_info, sizeof (exception_pc_line_info),
-              "0x%llx", (unsigned long long)exception_pc);
+	      "0x%llx", (unsigned long long)exception_pc);
 
   if (divergent)
-    gdb_printf (
-        _ ("Autostep caught exception at instruction before 0x%llx\n"
-           "This is probably %s\n"),
-        (unsigned long long)after_pc, exception_pc_line_info);
+    gdb_printf (_ ("Autostep caught exception at instruction before 0x%llx\n"
+		   "This is probably %s\n"),
+		(unsigned long long)after_pc, exception_pc_line_info);
   else
     gdb_printf (_ ("Autostep precisely caught exception at %s\n"),
-                     exception_pc_line_info);
+		exception_pc_line_info);
 
   set_internalvar (lookup_internalvar ("autostep_exception_pc"),
-                   value_from_longest (type_data_ptr, (LONGEST)exception_pc));
+		   value_from_longest (type_data_ptr, (LONGEST)exception_pc));
   set_internalvar (
       lookup_internalvar ("autostep_exception_line"),
       value_from_longest (type_uint32, (LONGEST)exception_sal.line));
@@ -185,10 +184,11 @@ find_end_pc (uint64_t pc)
   if (bl)
     return bl->end ();
 
-  struct bound_minimal_symbol msymbol = lookup_minimal_symbol_by_pc ((CORE_ADDR)pc);
+  struct bound_minimal_symbol msymbol
+      = lookup_minimal_symbol_by_pc ((CORE_ADDR)pc);
   if (msymbol.minsym)
-    return  (uint64_t)msymbol.minsym->unrelocated_address ()
-           + (uint64_t)msymbol.minsym->size ();
+    return (uint64_t)msymbol.minsym->unrelocated_address ()
+	   + (uint64_t)msymbol.minsym->size ();
 
   return (uint64_t)-1LL;
 }
@@ -205,14 +205,14 @@ count_instructions (uint64_t pc, uint64_t end_pc)
   gdb_assert (disassembler);
 
   const uint32_t inst_size = disassembler->insn_size ();
-  
+
   cuda_adjust_device_code_address (pc, &pc);
   for (; pc < end_pc; pc += inst_size)
     {
       auto inst = disassembler->disassemble_instruction (pc);
       /* Stop counting if pc is outside of the routine boundary */
       if (!inst)
-        return count;
+	return count;
       ++count;
     }
 
@@ -232,9 +232,9 @@ count_lines (uint64_t pc, uint64_t end_pc, uint32_t inst_size)
       /* Check if line numbers differ.
        * If no line information exists treat each instruction as one line. */
       if (!(cur_sal.symtab && cur_sal.line)
-          || !(next_sal.symtab && next_sal.line)
-          || cur_sal.line != next_sal.line)
-        ++nlines;
+	  || !(next_sal.symtab && next_sal.line)
+	  || cur_sal.line != next_sal.line)
+	++nlines;
     }
 
   return nlines;
@@ -274,8 +274,6 @@ initialize_autostep_state (CORE_ADDR astep_pc)
 static bool
 astep_warp_valid_p (cuda_coords &coords)
 {
-  /* Ensure the physical coords are up-to-date */
-  coords.resetPhysical ();
   if (!coords.valid ())
     return false;
 
@@ -309,7 +307,7 @@ select_next_valid_warp (void)
   cuda_coords coord{ astep_state.device.cur_coords };
   cuda_coords next_coord{ coord };
 
-  if (iter_pos == iter.end ())
+  if ((iter_pos == iter.end ()) || (++iter_pos == iter.end ()))
     return false;
 
   cuda_trace_domain (
@@ -320,69 +318,69 @@ select_next_valid_warp (void)
       coord.logical ().threadIdx ().y, coord.logical ().blockIdx ().x,
       coord.logical ().blockIdx ().y, coord.logical ().blockIdx ().z);
 
-  /* Skip to next warp (by using possibly outdated physical coordinates,
-     but sorted correctly by logical coordinates) */
+  /* Skip to next warp */
   do
     {
       /* Set next_coord to the next coord in the iterator */
-      next_coord = *(++iter_pos);
+      next_coord = *iter_pos;
 
       /* This will skip over other lanes in the current warp as they
        * will have advanced past the start pc */
       if (astep_warp_valid_p (next_coord))
-        {
-          /* Check to see if we need to update the threadIdx, we want
-           * to set focus to the lowest numbered lane. */
-          uint32_t ln = cuda_state::warp_get_lowest_active_lane (
-              next_coord.physical ().dev (), next_coord.physical ().sm (),
-              next_coord.physical ().wp ());
-          if (next_coord.physical ().ln () != ln)
-            {
-              /* Search for threadIdx */
-              cuda_coords filter{ next_coord.physical ().dev (),
-                                  next_coord.physical ().sm (),
-                                  next_coord.physical ().wp (),
-                                  ln,
-                                  next_coord.logical ().kernelId (),
-                                  next_coord.logical ().gridId (),
-                                  next_coord.logical ().clusterIdx (),
-                                  next_coord.logical ().blockIdx (),
-                                  CUDA_WILDCARD_DIM };
-              cuda_coord_set<cuda_coord_set_type::threads,
-                            select_valid | select_sngl>
-                  lane{ filter };
-              gdb_assert (lane.size ());
-              next_coord = *lane.begin ();
-            }
+	{
+	  /* Check to see if we need to update the threadIdx, we want
+	   * to set focus to the lowest numbered lane. */
+	  uint32_t ln = cuda_state::warp_get_lowest_active_lane (
+	      next_coord.physical ().dev (), next_coord.physical ().sm (),
+	      next_coord.physical ().wp ());
+	  if (next_coord.physical ().ln () != ln)
+	    {
+	      /* Search for threadIdx */
+	      cuda_coords filter{ next_coord.physical ().dev (),
+				  next_coord.physical ().sm (),
+				  next_coord.physical ().wp (),
+				  ln,
+				  next_coord.logical ().kernelId (),
+				  next_coord.logical ().gridId (),
+				  next_coord.logical ().clusterIdx (),
+				  next_coord.logical ().clusterDim (),
+				  next_coord.logical ().blockIdx (),
+				  CUDA_WILDCARD_DIM };
+	      cuda_coord_set<cuda_coord_set_type::threads,
+			     select_valid | select_sngl>
+		  lane{ filter };
+	      gdb_assert (lane.size ());
+	      next_coord = *lane.begin ();
+	    }
 	  switch_to_cuda_thread (next_coord);
 
-          const auto &p = next_coord.physical ();
-          const auto &l = next_coord.logical ();
-          CORE_ADDR warp_pc = cuda_state::warp_get_active_pc (
-              p.dev (), p.sm (), p.wp ());
+	  const auto &p = next_coord.physical ();
+	  const auto &l = next_coord.logical ();
+	  CORE_ADDR warp_pc
+	      = cuda_state::warp_get_active_pc (p.dev (), p.sm (), p.wp ());
 
-          cuda_trace_domain (
-              CUDA_TRACE_BREAKPOINT,
-              "Autostep: next warp: tId=(%d,%d,%d) bId=(%d,%d,%d)",
-              l.threadIdx ().x, l.threadIdx ().y, l.threadIdx ().y,
-              l.blockIdx ().x, l.blockIdx ().y, l.blockIdx ().z);
+	  cuda_trace_domain (
+	      CUDA_TRACE_BREAKPOINT,
+	      "Autostep: next warp: tId=(%d,%d,%d) bId=(%d,%d,%d)",
+	      l.threadIdx ().x, l.threadIdx ().y, l.threadIdx ().y,
+	      l.blockIdx ().x, l.blockIdx ().y, l.blockIdx ().z);
 
-          /* Now that we have found a valid warp, reset the autostep state
-             to reflect the region this warp will autostep through.  */
-          auto iter_cpy = iter;
-          auto iter_pos_cpy = iter_pos;
-          initialize_autostep_state (warp_pc);
-          astep_state.device.cur_coords = next_coord;
-          astep_state.device.iter = iter_cpy;
-          astep_state.device.iter_pos = iter_pos_cpy;
-          astep_state.device.cur_ln = ln;
-          return true;
-        }
+	  /* Now that we have found a valid warp, reset the autostep state
+	     to reflect the region this warp will autostep through.  */
+	  auto iter_cpy = iter;
+	  auto iter_pos_cpy = iter_pos;
+	  initialize_autostep_state (warp_pc);
+	  astep_state.device.cur_coords = next_coord;
+	  astep_state.device.iter = iter_cpy;
+	  astep_state.device.iter_pos = iter_pos_cpy;
+	  astep_state.device.cur_ln = ln;
+	  return true;
+	}
     }
-  while (iter_pos != iter.end ()
-         && coord.physical ().dev () == next_coord.physical ().dev ()
-         && coord.physical ().sm () == next_coord.physical ().sm ()
-         && coord.physical ().wp () == next_coord.physical ().wp ());
+  while ((++iter_pos) != iter.end ()
+	 && coord.physical ().dev () == next_coord.physical ().dev ()
+	 && coord.physical ().sm () == next_coord.physical ().sm ()
+	 && coord.physical ().wp () == next_coord.physical ().wp ());
 
   return false;
 }
@@ -402,8 +400,8 @@ set_next_device_iteration (void)
 
   /* A valid warp is already in place and we are starting to step this warp
      from the start.  */
-  uint64_t cur_pc = (CORE_ADDR)cuda_state::warp_get_active_pc (
-      p.dev (), p.sm (), p.wp ());
+  uint64_t cur_pc
+      = (CORE_ADDR)cuda_state::warp_get_active_pc (p.dev (), p.sm (), p.wp ());
   struct symtab_and_line cur_sal = find_pc_line (cur_pc, 0);
   uint64_t end_pc = -1;
 
@@ -425,13 +423,13 @@ set_next_device_iteration (void)
        * corresponding to limiting line. */
 
       std::vector<CORE_ADDR> line_pcs = find_pcs_for_symtab_line (
-          cur_sal.symtab, cur_sal.line + remaining, &best_item);
+	  cur_sal.symtab, cur_sal.line + remaining, &best_item);
 
       for (CORE_ADDR &line_pc : line_pcs)
-        {
-          if (cur_pc < line_pc && line_pc < end_pc)
-            end_pc = line_pc;
-        }
+	{
+	  if (cur_pc < line_pc && line_pc < end_pc)
+	    end_pc = line_pc;
+	}
     }
 
   /* If pc is in the top frame - do not allow autostepping outside of kernel
@@ -441,22 +439,22 @@ set_next_device_iteration (void)
       uint64_t kernel_end_pc;
       kernel_end_pc = find_end_pc (cur_pc);
       if (cur_pc >= kernel_end_pc)
-        {
-          /* STOP AUTOSTEPPING!!!!!! */
-          return 1;
-        }
+	{
+	  /* STOP AUTOSTEPPING!!!!!! */
+	  return 1;
+	}
       if (kernel_end_pc < end_pc)
-        end_pc = kernel_end_pc;
+	end_pc = kernel_end_pc;
     }
 
   /* Calculate how many steps should be taken */
   if (cuda_options_single_stepping_optimizations_enabled ())
     {
       auto found = cuda_find_next_control_flow_instruction (
-          cur_pc, cur_sal.pc, end_pc, false, end_pc, inst_size);
+	  cur_pc, cur_sal.pc, end_pc, false, end_pc, inst_size);
       /* Stop autostepping if instruction scanning failed */
       if (!found)
-        return 1;
+	return 1;
     }
   /* This ensures we are not at a control flow instruction */
   if (end_pc != cur_pc)
@@ -489,11 +487,10 @@ static int
 cuda_initialize_device_autostep (CORE_ADDR pc)
 {
   /* Iterate through all warps in current grid that are at a breakpoint */
-  cuda_coords filter{
-    CUDA_CURRENT,      CUDA_WILDCARD,     CUDA_WILDCARD,
-    CUDA_WILDCARD,     CUDA_WILDCARD,     CUDA_CURRENT,
-    CUDA_WILDCARD_DIM, CUDA_WILDCARD_DIM, CUDA_WILDCARD_DIM
-  };
+  cuda_coords filter{ CUDA_CURRENT,	 CUDA_WILDCARD,	    CUDA_WILDCARD,
+		      CUDA_WILDCARD,	 CUDA_WILDCARD,	    CUDA_CURRENT,
+		      CUDA_WILDCARD_DIM, CUDA_WILDCARD_DIM, CUDA_WILDCARD_DIM,
+		      CUDA_WILDCARD_DIM };
   cuda_coord_set<cuda_coord_set_type::threads, select_bkpt | select_valid> itr{
     filter, {}
   };
@@ -506,6 +503,7 @@ cuda_initialize_device_autostep (CORE_ADDR pc)
   if (!astep_warp_valid_p (astep_state.device.cur_coords))
     if (!select_next_valid_warp ())
       return 1;
+  switch_to_cuda_thread (astep_state.device.cur_coords);
 
   /* Now that we've got the basic things out of the way, set the first
    * iteration next.  */
@@ -536,7 +534,7 @@ cuda_initialize_autostep (CORE_ADDR pc)
   if (cuda_current_focus::isDevice ())
     {
       if (cuda_initialize_device_autostep (pc) != 0)
-        return 1;
+	return 1;
     }
   else
     cuda_initialize_host_autostep (pc);
@@ -568,11 +566,11 @@ update_host_autostep_state (CORE_ADDR pc)
     {
       /* We are instruction-stepping.  Move to the next instruction.  */
       if (pc != astep_state.cur_pc)
-        {
-          astep_state.insns_stepped++;
-          astep_state.remaining--;
-          astep_state.cur_pc = pc;
-        }
+	{
+	  astep_state.insns_stepped++;
+	  astep_state.remaining--;
+	  astep_state.cur_pc = pc;
+	}
     }
   else
     {
@@ -581,12 +579,12 @@ update_host_autostep_state (CORE_ADDR pc)
       sal = find_pc_line (pc, 0);
 
       if (sal.line != 0 && sal.line != astep_state.cur_sal.line)
-        {
-          astep_state.lines_stepped++;
-          astep_state.remaining--;
-          astep_state.cur_sal = sal;
-          astep_state.cur_pc = pc;
-        }
+	{
+	  astep_state.lines_stepped++;
+	  astep_state.remaining--;
+	  astep_state.cur_sal = sal;
+	  astep_state.cur_pc = pc;
+	}
     }
 
   /* Check if we have an overlapping autostep region that we should ignore.  */
@@ -626,24 +624,23 @@ update_device_autostep_state (CORE_ADDR pc)
 
   /* Make sure we can continue stepping this warp */
   auto &c = astep_state.device.cur_coords;
-  c.resetPhysical ();
   if (!c.valid () || !cuda_current_focus::isDevice ())
     {
       /* It looks like this warp ran to completion or became invalid.  Switch
-         to the next valid one.  */
+	 to the next valid one.  */
       cuda_trace_domain (
-          CUDA_TRACE_BREAKPOINT,
-          "Autostep: warp complete! Previous was: tId=(%d,%d,%d) "
-          "bId=(%d,%d,%d)",
-          c.logical ().threadIdx ().x, c.logical ().threadIdx ().y,
-          c.logical ().threadIdx ().y, c.logical ().blockIdx ().x,
-          c.logical ().blockIdx ().y, c.logical ().blockIdx ().z);
+	  CUDA_TRACE_BREAKPOINT,
+	  "Autostep: warp complete! Previous was: tId=(%d,%d,%d) "
+	  "bId=(%d,%d,%d)",
+	  c.logical ().threadIdx ().x, c.logical ().threadIdx ().y,
+	  c.logical ().threadIdx ().y, c.logical ().blockIdx ().x,
+	  c.logical ().blockIdx ().y, c.logical ().blockIdx ().z);
 
       if (!select_next_valid_warp ())
-        return 1;
+	return 1;
 
       /* Set things up to we can autostep the warp that is currently
-         selected.  */
+	 selected.  */
       set_next_device_iteration ();
       return 0;
     }
@@ -682,22 +679,22 @@ update_device_autostep_state (CORE_ADDR pc)
   if (nsteps > 1)
     {
       /* We were supposed to instruction-step multiple instructions at
-         once.  */
+	 once.  */
       nsteps = count_instructions (before_pc, after_pc);
       lines = count_lines (before_pc, after_pc, inst_size);
     }
   else /* Control flow instruction */
     {
       /* We were sitting at a control flow instruction, so we needed to
-         instruction-step only once to see where that instruction
-         would take us.  */
+	 instruction-step only once to see where that instruction
+	 would take us.  */
       gdb_assert (nsteps == 1);
 
       /* Calculate lines - if no line information exists treat it as one
-         instruction */
+	 instruction */
       lines = !(before_sal.symtab && before_sal.line)
-              || !(after_sal.symtab && after_sal.line)
-              || before_sal.line != after_sal.line;
+	      || !(after_sal.symtab && after_sal.line)
+	      || before_sal.line != after_sal.line;
     }
 
   /* Update the number of remaining instructions/lines we must step through. */
@@ -720,30 +717,30 @@ update_device_autostep_state (CORE_ADDR pc)
   if (remaining <= 0)
     {
       /* We are done with the autostep range.  Check if there is an adjacent
-         autostep region we should handle.  */
+	 autostep region we should handle.  */
       if (overlap)
-        {
-          /* There is an adjacent autostep region.  Proceed to handle it
-             without switching focus to a different warp.  We will get back to
-             the other warps later on.  */
+	{
+	  /* There is an adjacent autostep region.  Proceed to handle it
+	     without switching focus to a different warp.  We will get back to
+	     the other warps later on.  */
 	  auto old_coord = c;
-          auto iter = astep_state.device.iter;
-          auto iter_pos = astep_state.device.iter_pos;
-          initialize_autostep_state (after_pc);
-          astep_state.device.cur_coords = old_coord;
-          astep_state.device.iter = iter;
-          astep_state.device.iter_pos = iter_pos;
-        }
+	  auto iter = astep_state.device.iter;
+	  auto iter_pos = astep_state.device.iter_pos;
+	  initialize_autostep_state (after_pc);
+	  astep_state.device.cur_coords = old_coord;
+	  astep_state.device.iter = iter;
+	  astep_state.device.iter_pos = iter_pos;
+	}
       else
-        {
-          /* We are done with this autostep region and there are no adjacent
-             autostep regions to handle.  Just switch to the next valid warp
-             and set it up so we can autostep it through the autostep
-             region.  */
-          /* Select the next valid warp.  */
-          if (!select_next_valid_warp ())
-            return 1;
-        }
+	{
+	  /* We are done with this autostep region and there are no adjacent
+	     autostep regions to handle.  Just switch to the next valid warp
+	     and set it up so we can autostep it through the autostep
+	     region.  */
+	  /* Select the next valid warp.  */
+	  if (!select_next_valid_warp ())
+	    return 1;
+	}
     }
   else
     {
@@ -824,11 +821,11 @@ cuda_autostep_print_exception (void)
     {
       /* This is an exception */
       if (cuda_current_focus::isDevice ())
-        autostep_report_exception_device (astep_state.device.nsteps,
-                                          astep_state.device.cur_ln,
-                                          astep_state.cur_pc, tp->stop_pc ());
+	autostep_report_exception_device (astep_state.device.nsteps,
+					  astep_state.device.cur_ln,
+					  astep_state.cur_pc, tp->stop_pc ());
       else
-        autostep_report_exception_host (astep_state.cur_pc);
+	autostep_report_exception_host (astep_state.cur_pc);
 
       /* We are done with autostepping.  */
       cuda_cleanup_autostep_state ();
