@@ -889,6 +889,9 @@ public:
 #ifdef NVIDIA_CUDA_GDB
   /* CUDA - cuda_objfile */
   bool cuda_objfile = false;
+  /* CUDA - line-table-only debug info (compiled with -lineinfo, no
+   * .debug_info). */
+  bool cuda_line_table_only = false;
   /* CUDA - Whether this objfile is in the process of being discarded or not.
      This is used as a way to tell GDB that we no longer want types associated
      with this particular CUDA objfile and therefore want the types to be
