@@ -43,11 +43,6 @@ typedef struct cuDeviceGetName_params_st {
     CUdevice dev;
 } cuDeviceGetName_params;
 
-typedef struct cuDeviceGetUuid_params_st {
-    CUuuid *uuid;
-    CUdevice dev;
-} cuDeviceGetUuid_params;
-
 typedef struct cuDeviceGetUuid_v2_params_st {
     CUuuid *uuid;
     CUdevice dev;
@@ -144,20 +139,6 @@ typedef struct cuDevicePrimaryCtxReset_v2_params_st {
     CUdevice dev;
 } cuDevicePrimaryCtxReset_v2_params;
 
-typedef struct cuCtxCreate_v2_params_st {
-    CUcontext *pctx;
-    unsigned int flags;
-    CUdevice dev;
-} cuCtxCreate_v2_params;
-
-typedef struct cuCtxCreate_v3_params_st {
-    CUcontext *pctx;
-    CUexecAffinityParam *paramsArray;
-    int numParams;
-    unsigned int flags;
-    CUdevice dev;
-} cuCtxCreate_v3_params;
-
 typedef struct cuCtxCreate_v4_params_st {
     CUcontext *pctx;
     CUctxCreateParams *ctxCreateParams;
@@ -703,7 +684,7 @@ typedef struct cuMemcpy3DPeerAsync_ptsz_params_st {
     CUstream hStream;
 } cuMemcpy3DPeerAsync_ptsz_params;
 
-typedef struct cuMemcpyBatchAsync_ptsz_params_st {
+typedef struct cuMemcpyBatchAsync_v2_ptsz_params_st {
     CUdeviceptr *dsts;
     CUdeviceptr *srcs;
     size_t *sizes;
@@ -711,17 +692,15 @@ typedef struct cuMemcpyBatchAsync_ptsz_params_st {
     CUmemcpyAttributes *attrs;
     size_t *attrsIdxs;
     size_t numAttrs;
-    size_t *failIdx;
     CUstream hStream;
-} cuMemcpyBatchAsync_ptsz_params;
+} cuMemcpyBatchAsync_v2_ptsz_params;
 
-typedef struct cuMemcpy3DBatchAsync_ptsz_params_st {
+typedef struct cuMemcpy3DBatchAsync_v2_ptsz_params_st {
     size_t numOps;
     CUDA_MEMCPY3D_BATCH_OP *opList;
-    size_t *failIdx;
     unsigned long long flags;
     CUstream hStream;
-} cuMemcpy3DBatchAsync_ptsz_params;
+} cuMemcpy3DBatchAsync_v2_ptsz_params;
 
 typedef struct cuMemsetD8_v2_ptds_params_st {
     CUdeviceptr dstDevice;
@@ -1061,13 +1040,6 @@ typedef struct cuPointerGetAttribute_params_st {
     CUdeviceptr ptr;
 } cuPointerGetAttribute_params;
 
-typedef struct cuMemPrefetchAsync_ptsz_params_st {
-    CUdeviceptr devPtr;
-    size_t count;
-    CUdevice dstDevice;
-    CUstream hStream;
-} cuMemPrefetchAsync_ptsz_params;
-
 typedef struct cuMemPrefetchAsync_v2_ptsz_params_st {
     CUdeviceptr devPtr;
     size_t count;
@@ -1076,13 +1048,6 @@ typedef struct cuMemPrefetchAsync_v2_ptsz_params_st {
     CUstream hStream;
 } cuMemPrefetchAsync_v2_ptsz_params;
 
-typedef struct cuMemAdvise_params_st {
-    CUdeviceptr devPtr;
-    size_t count;
-    CUmem_advise advice;
-    CUdevice device;
-} cuMemAdvise_params;
-
 typedef struct cuMemAdvise_v2_params_st {
     CUdeviceptr devPtr;
     size_t count;
@@ -1198,15 +1163,6 @@ typedef struct cuStreamIsCapturing_ptsz_params_st {
     CUstreamCaptureStatus *captureStatus;
 } cuStreamIsCapturing_ptsz_params;
 
-typedef struct cuStreamGetCaptureInfo_v2_ptsz_params_st {
-    CUstream hStream;
-    CUstreamCaptureStatus *captureStatus_out;
-    cuuint64_t *id_out;
-    CUgraph *graph_out;
-    const CUgraphNode **dependencies_out;
-    size_t *numDependencies_out;
-} cuStreamGetCaptureInfo_v2_ptsz_params;
-
 typedef struct cuStreamGetCaptureInfo_v3_ptsz_params_st {
     CUstream hStream;
     CUstreamCaptureStatus *captureStatus_out;
@@ -1217,13 +1173,6 @@ typedef struct cuStreamGetCaptureInfo_v3_ptsz_params_st {
     size_t *numDependencies_out;
 } cuStreamGetCaptureInfo_v3_ptsz_params;
 
-typedef struct cuStreamUpdateCaptureDependencies_ptsz_params_st {
-    CUstream hStream;
-    CUgraphNode *dependencies;
-    size_t numDependencies;
-    unsigned int flags;
-} cuStreamUpdateCaptureDependencies_ptsz_params;
-
 typedef struct cuStreamUpdateCaptureDependencies_v2_ptsz_params_st {
     CUstream hStream;
     CUgraphNode *dependencies;
@@ -1785,13 +1734,6 @@ typedef struct cuGraphGetRootNodes_params_st {
     size_t *numRootNodes;
 } cuGraphGetRootNodes_params;
 
-typedef struct cuGraphGetEdges_params_st {
-    CUgraph hGraph;
-    CUgraphNode *from;
-    CUgraphNode *to;
-    size_t *numEdges;
-} cuGraphGetEdges_params;
-
 typedef struct cuGraphGetEdges_v2_params_st {
     CUgraph hGraph;
     CUgraphNode *from;
@@ -1800,12 +1742,6 @@ typedef struct cuGraphGetEdges_v2_params_st {
     size_t *numEdges;
 } cuGraphGetEdges_v2_params;
 
-typedef struct cuGraphNodeGetDependencies_params_st {
-    CUgraphNode hNode;
-    CUgraphNode *dependencies;
-    size_t *numDependencies;
-} cuGraphNodeGetDependencies_params;
-
 typedef struct cuGraphNodeGetDependencies_v2_params_st {
     CUgraphNode hNode;
     CUgraphNode *dependencies;
@@ -1813,12 +1749,6 @@ typedef struct cuGraphNodeGetDependencies_v2_params_st {
     size_t *numDependencies;
 } cuGraphNodeGetDependencies_v2_params;
 
-typedef struct cuGraphNodeGetDependentNodes_params_st {
-    CUgraphNode hNode;
-    CUgraphNode *dependentNodes;
-    size_t *numDependentNodes;
-} cuGraphNodeGetDependentNodes_params;
-
 typedef struct cuGraphNodeGetDependentNodes_v2_params_st {
     CUgraphNode hNode;
     CUgraphNode *dependentNodes;
@@ -1826,13 +1756,6 @@ typedef struct cuGraphNodeGetDependentNodes_v2_params_st {
     size_t *numDependentNodes;
 } cuGraphNodeGetDependentNodes_v2_params;
 
-typedef struct cuGraphAddDependencies_params_st {
-    CUgraph hGraph;
-    const CUgraphNode *from;
-    const CUgraphNode *to;
-    size_t numDependencies;
-} cuGraphAddDependencies_params;
-
 typedef struct cuGraphAddDependencies_v2_params_st {
     CUgraph hGraph;
     const CUgraphNode *from;
@@ -1841,13 +1764,6 @@ typedef struct cuGraphAddDependencies_v2_params_st {
     size_t numDependencies;
 } cuGraphAddDependencies_v2_params;
 
-typedef struct cuGraphRemoveDependencies_params_st {
-    CUgraph hGraph;
-    const CUgraphNode *from;
-    const CUgraphNode *to;
-    size_t numDependencies;
-} cuGraphRemoveDependencies_params;
-
 typedef struct cuGraphRemoveDependencies_v2_params_st {
     CUgraph hGraph;
     const CUgraphNode *from;
@@ -1860,14 +1776,6 @@ typedef struct cuGraphDestroyNode_params_st {
     CUgraphNode hNode;
 } cuGraphDestroyNode_params;
 
-typedef struct cuGraphInstantiate_v2_params_st {
-    CUgraphExec *phGraphExec;
-    CUgraph hGraph;
-    CUgraphNode *phErrorNode;
-    char *logBuffer;
-    size_t bufferSize;
-} cuGraphInstantiate_v2_params;
-
 typedef struct cuGraphInstantiateWithFlags_params_st {
     CUgraphExec *phGraphExec;
     CUgraph hGraph;
@@ -2031,14 +1939,6 @@ typedef struct cuGraphReleaseUserObject_params_st {
     unsigned int count;
 } cuGraphReleaseUserObject_params;
 
-typedef struct cuGraphAddNode_params_st {
-    CUgraphNode *phGraphNode;
-    CUgraph hGraph;
-    const CUgraphNode *dependencies;
-    size_t numDependencies;
-    CUgraphNodeParams *nodeParams;
-} cuGraphAddNode_params;
-
 typedef struct cuGraphAddNode_v2_params_st {
     CUgraphNode *phGraphNode;
     CUgraph hGraph;
@@ -2541,6 +2441,20 @@ typedef struct cuCtxCreate_params_st {
     CUdevice dev;
 } cuCtxCreate_params;
 
+typedef struct cuCtxCreate_v2_params_st {
+    CUcontext *pctx;
+    unsigned int flags;
+    CUdevice dev;
+} cuCtxCreate_v2_params;
+
+typedef struct cuCtxCreate_v3_params_st {
+    CUcontext *pctx;
+    CUexecAffinityParam *paramsArray;
+    int numParams;
+    unsigned int flags;
+    CUdevice dev;
+} cuCtxCreate_v3_params;
+
 typedef struct cuModuleGetGlobal_params_st {
     CUdeviceptr_v1 *dptr;
     unsigned int *bytes;
@@ -2820,6 +2734,11 @@ typedef struct cuDevicePrimaryCtxSetFlags_params_st {
     unsigned int flags;
 } cuDevicePrimaryCtxSetFlags_params;
 
+typedef struct cuDeviceGetUuid_params_st {
+    CUuuid *uuid;
+    CUdevice dev;
+} cuDeviceGetUuid_params;
+
 typedef struct cuMemcpyHtoD_v2_params_st {
     CUdeviceptr dstDevice;
     const void *srcHost;
@@ -3034,6 +2953,44 @@ typedef struct cuMemcpy3DBatchAsync_params_st {
     CUstream hStream;
 } cuMemcpy3DBatchAsync_params;
 
+typedef struct cuMemcpyBatchAsync_ptsz_params_st {
+    CUdeviceptr *dsts;
+    CUdeviceptr *srcs;
+    size_t *sizes;
+    size_t count;
+    CUmemcpyAttributes *attrs;
+    size_t *attrsIdxs;
+    size_t numAttrs;
+    size_t *failIdx;
+    CUstream hStream;
+} cuMemcpyBatchAsync_ptsz_params;
+
+typedef struct cuMemcpy3DBatchAsync_ptsz_params_st {
+    size_t numOps;
+    CUDA_MEMCPY3D_BATCH_OP *opList;
+    size_t *failIdx;
+    unsigned long long flags;
+    CUstream hStream;
+} cuMemcpy3DBatchAsync_ptsz_params;
+
+typedef struct cuMemcpyBatchAsync_v2_params_st {
+    CUdeviceptr *dsts;
+    CUdeviceptr *srcs;
+    size_t *sizes;
+    size_t count;
+    CUmemcpyAttributes *attrs;
+    size_t *attrsIdxs;
+    size_t numAttrs;
+    CUstream hStream;
+} cuMemcpyBatchAsync_v2_params;
+
+typedef struct cuMemcpy3DBatchAsync_v2_params_st {
+    size_t numOps;
+    CUDA_MEMCPY3D_BATCH_OP *opList;
+    unsigned long long flags;
+    CUstream hStream;
+} cuMemcpy3DBatchAsync_v2_params;
+
 typedef struct cuMemsetD8Async_params_st {
     CUdeviceptr dstDevice;
     unsigned char uc;
@@ -3298,6 +3255,13 @@ typedef struct cuMemPrefetchAsync_params_st {
     CUstream hStream;
 } cuMemPrefetchAsync_params;
 
+typedef struct cuMemPrefetchAsync_ptsz_params_st {
+    CUdeviceptr devPtr;
+    size_t count;
+    CUdevice dstDevice;
+    CUstream hStream;
+} cuMemPrefetchAsync_ptsz_params;
+
 typedef struct cuMemPrefetchAsync_v2_params_st {
     CUdeviceptr devPtr;
     size_t count;
@@ -3306,6 +3270,13 @@ typedef struct cuMemPrefetchAsync_v2_params_st {
     CUstream hStream;
 } cuMemPrefetchAsync_v2_params;
 
+typedef struct cuMemAdvise_params_st {
+    CUdeviceptr devPtr;
+    size_t count;
+    CUmem_advise advice;
+    CUdevice device;
+} cuMemAdvise_params;
+
 typedef struct cuLaunchCooperativeKernel_params_st {
     CUfunction f;
     unsigned int gridDimX;
@@ -3386,6 +3357,15 @@ typedef struct cuStreamGetCaptureInfo_v2_params_st {
     size_t *numDependencies_out;
 } cuStreamGetCaptureInfo_v2_params;
 
+typedef struct cuStreamGetCaptureInfo_v2_ptsz_params_st {
+    CUstream hStream;
+    CUstreamCaptureStatus *captureStatus_out;
+    cuuint64_t *id_out;
+    CUgraph *graph_out;
+    const CUgraphNode **dependencies_out;
+    size_t *numDependencies_out;
+} cuStreamGetCaptureInfo_v2_ptsz_params;
+
 typedef struct cuStreamGetCaptureInfo_v3_params_st {
     CUstream hStream;
     CUstreamCaptureStatus *captureStatus_out;
@@ -3443,6 +3423,47 @@ typedef struct cuGraphLaunch_params_st {
     CUstream hStream;
 } cuGraphLaunch_params;
 
+typedef struct cuGraphGetEdges_params_st {
+    CUgraph hGraph;
+    CUgraphNode *from;
+    CUgraphNode *to;
+    size_t *numEdges;
+} cuGraphGetEdges_params;
+
+typedef struct cuGraphNodeGetDependencies_params_st {
+    CUgraphNode hNode;
+    CUgraphNode *dependencies;
+    size_t *numDependencies;
+} cuGraphNodeGetDependencies_params;
+
+typedef struct cuGraphNodeGetDependentNodes_params_st {
+    CUgraphNode hNode;
+    CUgraphNode *dependentNodes;
+    size_t *numDependentNodes;
+} cuGraphNodeGetDependentNodes_params;
+
+typedef struct cuGraphAddDependencies_params_st {
+    CUgraph hGraph;
+    const CUgraphNode *from;
+    const CUgraphNode *to;
+    size_t numDependencies;
+} cuGraphAddDependencies_params;
+
+typedef struct cuGraphRemoveDependencies_params_st {
+    CUgraph hGraph;
+    const CUgraphNode *from;
+    const CUgraphNode *to;
+    size_t numDependencies;
+} cuGraphRemoveDependencies_params;
+
+typedef struct cuGraphAddNode_params_st {
+    CUgraphNode *phGraphNode;
+    CUgraph hGraph;
+    const CUgraphNode *dependencies;
+    size_t numDependencies;
+    CUgraphNodeParams *nodeParams;
+} cuGraphAddNode_params;
+
 typedef struct cuStreamCopyAttributes_params_st {
     CUstream dstStream;
     CUstream srcStream;
@@ -3474,6 +3495,14 @@ typedef struct cuGraphInstantiate_params_st {
     size_t bufferSize;
 } cuGraphInstantiate_params;
 
+typedef struct cuGraphInstantiate_v2_params_st {
+    CUgraphExec *phGraphExec;
+    CUgraph hGraph;
+    CUgraphNode *phErrorNode;
+    char *logBuffer;
+    size_t bufferSize;
+} cuGraphInstantiate_v2_params;
+
 typedef struct cuMemMapArrayAsync_params_st {
     CUarrayMapInfo *mapInfoList;
     unsigned int count;
@@ -3505,6 +3534,13 @@ typedef struct cuStreamUpdateCaptureDependencies_params_st {
     unsigned int flags;
 } cuStreamUpdateCaptureDependencies_params;
 
+typedef struct cuStreamUpdateCaptureDependencies_ptsz_params_st {
+    CUstream hStream;
+    CUgraphNode *dependencies;
+    size_t numDependencies;
+    unsigned int flags;
+} cuStreamUpdateCaptureDependencies_ptsz_params;
+
 typedef struct cuStreamUpdateCaptureDependencies_v2_params_st {
     CUstream hStream;
     CUgraphNode *dependencies;
