@@ -1542,6 +1542,31 @@ typedef struct cudaGraphNodeGetType_v10000_params_st {
     enum cudaGraphNodeType *pType;
 } cudaGraphNodeGetType_v10000_params;
 
+typedef struct cudaGraphNodeGetContainingGraph_v13010_params_st {
+    cudaGraphNode_t hNode;
+    cudaGraph_t *phGraph;
+} cudaGraphNodeGetContainingGraph_v13010_params;
+
+typedef struct cudaGraphNodeGetLocalId_v13010_params_st {
+    cudaGraphNode_t hNode;
+    unsigned int *nodeId;
+} cudaGraphNodeGetLocalId_v13010_params;
+
+typedef struct cudaGraphNodeGetToolsId_v13010_params_st {
+    cudaGraphNode_t hNode;
+    unsigned long long *toolsNodeId;
+} cudaGraphNodeGetToolsId_v13010_params;
+
+typedef struct cudaGraphGetId_v13010_params_st {
+    cudaGraph_t hGraph;
+    unsigned int *graphID;
+} cudaGraphGetId_v13010_params;
+
+typedef struct cudaGraphExecGetId_v13010_params_st {
+    cudaGraphExec_t hGraphExec;
+    unsigned int *graphID;
+} cudaGraphExecGetId_v13010_params;
+
 typedef struct cudaGraphGetNodes_v10000_params_st {
     cudaGraph_t graph;
     cudaGraphNode_t *nodes;
@@ -1802,6 +1827,14 @@ typedef struct cudaGraphConditionalHandleCreate_v12030_params_st {
     unsigned int flags;
 } cudaGraphConditionalHandleCreate_v12030_params;
 
+typedef struct cudaGraphConditionalHandleCreate_v2_v13010_params_st {
+    cudaGraphConditionalHandle *pHandle_out;
+    cudaGraph_t graph;
+    cudaExecutionContext_t ctx;
+    unsigned int defaultLaunchValue;
+    unsigned int flags;
+} cudaGraphConditionalHandleCreate_v2_v13010_params;
+
 typedef struct cudaGetDriverEntryPoint_ptsz_v11030_params_st {
     const char *symbol;
     void **funcPtr;
@@ -1887,6 +1920,95 @@ typedef struct cudaKernelSetAttributeForDevice_v12060_params_st {
     int device;
 } cudaKernelSetAttributeForDevice_v12060_params;
 
+typedef struct cudaDeviceGetDevResource_v13010_params_st {
+    int device;
+    cudaDevResource *resource;
+    enum cudaDevResourceType type;
+} cudaDeviceGetDevResource_v13010_params;
+
+typedef struct cudaDevSmResourceSplitByCount_v13010_params_st {
+    cudaDevResource *result;
+    unsigned int *nbGroups;
+    const cudaDevResource *input;
+    cudaDevResource *remaining;
+    unsigned int useFlags;
+    unsigned int minCount;
+} cudaDevSmResourceSplitByCount_v13010_params;
+
+typedef struct cudaDevSmResourceSplit_v13010_params_st {
+    cudaDevResource *result;
+    unsigned int nbGroups;
+    const cudaDevResource *input;
+    cudaDevResource *remainder;
+    unsigned int flags;
+    cudaDevSmResourceGroupParams *groupParams;
+} cudaDevSmResourceSplit_v13010_params;
+
+typedef struct cudaDevResourceGenerateDesc_v13010_params_st {
+    cudaDevResourceDesc_t *phDesc;
+    cudaDevResource *resources;
+    unsigned int nbResources;
+} cudaDevResourceGenerateDesc_v13010_params;
+
+typedef struct cudaGreenCtxCreate_v13010_params_st {
+    cudaExecutionContext_t *phCtx;
+    cudaDevResourceDesc_t desc;
+    int device;
+    unsigned int flags;
+} cudaGreenCtxCreate_v13010_params;
+
+typedef struct cudaExecutionCtxDestroy_v13010_params_st {
+    cudaExecutionContext_t ctx;
+} cudaExecutionCtxDestroy_v13010_params;
+
+typedef struct cudaExecutionCtxGetDevResource_v13010_params_st {
+    cudaExecutionContext_t ctx;
+    cudaDevResource *resource;
+    enum cudaDevResourceType type;
+} cudaExecutionCtxGetDevResource_v13010_params;
+
+typedef struct cudaExecutionCtxGetDevice_v13010_params_st {
+    int *device;
+    cudaExecutionContext_t ctx;
+} cudaExecutionCtxGetDevice_v13010_params;
+
+typedef struct cudaExecutionCtxGetId_v13010_params_st {
+    cudaExecutionContext_t ctx;
+    unsigned long long *ctxId;
+} cudaExecutionCtxGetId_v13010_params;
+
+typedef struct cudaExecutionCtxStreamCreate_v13010_params_st {
+    cudaStream_t *phStream;
+    cudaExecutionContext_t ctx;
+    unsigned int flags;
+    int priority;
+} cudaExecutionCtxStreamCreate_v13010_params;
+
+typedef struct cudaExecutionCtxSynchronize_v13010_params_st {
+    cudaExecutionContext_t ctx;
+} cudaExecutionCtxSynchronize_v13010_params;
+
+typedef struct cudaStreamGetDevResource_ptsz_v13010_params_st {
+    cudaStream_t hStream;
+    cudaDevResource *resource;
+    enum cudaDevResourceType type;
+} cudaStreamGetDevResource_ptsz_v13010_params;
+
+typedef struct cudaExecutionCtxRecordEvent_v13010_params_st {
+    cudaExecutionContext_t ctx;
+    cudaEvent_t event;
+} cudaExecutionCtxRecordEvent_v13010_params;
+
+typedef struct cudaExecutionCtxWaitEvent_v13010_params_st {
+    cudaExecutionContext_t ctx;
+    cudaEvent_t event;
+} cudaExecutionCtxWaitEvent_v13010_params;
+
+typedef struct cudaDeviceGetExecutionCtx_v13010_params_st {
+    cudaExecutionContext_t *ctx;
+    int device;
+} cudaDeviceGetExecutionCtx_v13010_params;
+
 typedef struct cudaGetFuncBySymbol_v11000_params_st {
     cudaFunction_t *functionPtr;
     const void *symbolPtr;
@@ -2162,6 +2284,12 @@ typedef struct cudaStreamGetDevice_v12080_params_st {
     int *device;
 } cudaStreamGetDevice_v12080_params;
 
+typedef struct cudaStreamGetDevResource_v13010_params_st {
+    cudaStream_t hStream;
+    cudaDevResource *resource;
+    enum cudaDevResourceType type;
+} cudaStreamGetDevResource_v13010_params;
+
 typedef struct cudaStreamGetFlags_v5050_params_st {
     cudaStream_t hStream;
     unsigned int *flags;
