@@ -834,7 +834,30 @@ typedef enum SANITIZER_CBID_DRIVER_API_enum {
    SANITIZER_CBID_DRIVER_API_cuMulticastBindMem_v2                                        = 820,
    SANITIZER_CBID_DRIVER_API_cuMulticastBindAddr_v2                                       = 821,
    SANITIZER_CBID_DRIVER_API_cuDevSmResourceSplit                                         = 822,
-   SANITIZER_CBID_DRIVER_API_SIZE                                                         = 823,
+   SANITIZER_CBID_DRIVER_API_cuSubgridScheduleCreate                                      = 823,
+   SANITIZER_CBID_DRIVER_API_cuSubgridWorkerGridCreate                                    = 824,
+   SANITIZER_CBID_DRIVER_API_cuSubgridWorksetCreate                                       = 825,
+   SANITIZER_CBID_DRIVER_API_cuSubgridCreate                                              = 826,
+   SANITIZER_CBID_DRIVER_API_cuSubgridScheduleFinalize                                    = 827,
+   SANITIZER_CBID_DRIVER_API_cuSubgridScheduleLaunch                                      = 828,
+   SANITIZER_CBID_DRIVER_API_cuSubgridScheduleLaunch_ptsz                                 = 829,
+   SANITIZER_CBID_DRIVER_API_cuSubgridScheduleDestroy                                     = 830,
+   SANITIZER_CBID_DRIVER_API_cuLaunchHostFunc_v2                                          = 831,
+   SANITIZER_CBID_DRIVER_API_cuLaunchHostFunc_v2_ptsz                                     = 832,
+   SANITIZER_CBID_DRIVER_API_cuFuncGetDeviceCodeInfo                                      = 833,
+   SANITIZER_CBID_DRIVER_API_cuKernelGetDeviceCodeInfo                                    = 834,
+   SANITIZER_CBID_DRIVER_API_cuFuncGetParamCount                                          = 835,
+   SANITIZER_CBID_DRIVER_API_cuKernelGetParamCount                                        = 836,
+   SANITIZER_CBID_DRIVER_API_cuMemcpyWithAttributesAsync                                  = 837,
+   SANITIZER_CBID_DRIVER_API_cuMemcpyWithAttributesAsync_ptsz                             = 838,
+   SANITIZER_CBID_DRIVER_API_cuMemcpy3DWithAttributesAsync                                = 839,
+   SANITIZER_CBID_DRIVER_API_cuMemcpy3DWithAttributesAsync_ptsz                           = 840,
+   SANITIZER_CBID_DRIVER_API_cuGraphNodeGetParams                                         = 841,
+   SANITIZER_CBID_DRIVER_API_cuCoredumpRegisterStartCallback                              = 842,
+   SANITIZER_CBID_DRIVER_API_cuCoredumpRegisterCompleteCallback                           = 843,
+   SANITIZER_CBID_DRIVER_API_cuCoredumpDeregisterStartCallback                            = 844,
+   SANITIZER_CBID_DRIVER_API_cuCoredumpDeregisterCompleteCallback                         = 845,
+   SANITIZER_CBID_DRIVER_API_SIZE                                                         = 846,
    SANITIZER_CBID_DRIVER_API_FORCE_INT                                                    = 0x7fffffff,
 } SANITIZER_CBID_DRIVER_API;
 
