@@ -16,9 +16,10 @@
 #include <algorithm>
 #include <iomanip>
 #include <iostream>
-#include <memory>
+#include <string>
 #include <vector>
 
+#include <cupti_pmsampling.h>
 #include <cupti_profiler_host.h>
 #include <cupti_target.h>
 #include <cupti_range_profiler.h>
@@ -123,6 +124,21 @@ public:
         return CUPTI_SUCCESS;
     }
 
+    static CUptiResult GetCounterAvailabilityImageForPmSampling(
+        uint32_t deviceIndex,
+        std::vector<uint8_t>& counterAvailabilityImage
+    )
+    {
+        CUpti_PmSampling_GetCounterAvailability_Params getCounterAvailabilityParams { CUpti_PmSampling_GetCounterAvailability_Params_STRUCT_SIZE };
+        getCounterAvailabilityParams.deviceIndex = deviceIndex;
+        CUPTI_API_CALL(cuptiPmSamplingGetCounterAvailability(&getCounterAvailabilityParams));
+
+        counterAvailabilityImage.resize(getCounterAvailabilityParams.counterAvailabilityImageSize);
+        getCounterAvailabilityParams.pCounterAvailabilityImage = counterAvailabilityImage.data();
+        CUPTI_API_CALL(cuptiPmSamplingGetCounterAvailability(&getCounterAvailabilityParams));
+        return CUPTI_SUCCESS;
+    }
+
 protected:
 
     std::string m_chipName;
@@ -270,6 +286,7 @@ public:
         std::string& hwUnit,
         std::string& dimUnits,
         std::string& numPasses,
+        std::string& collectionScope,
         bool listNumPasses = false
     )
     {
@@ -286,6 +303,7 @@ public:
         description = hostGetMetricPropertiesParams.pDescription;
         hwUnit = hostGetMetricPropertiesParams.pHwUnit;
         dimUnits = hostGetMetricPropertiesParams.pDimUnit;
+        collectionScope = hostGetMetricPropertiesParams.metricCollectionScope == CUPTI_METRIC_COLLECTION_SCOPE_DEVICE ? "Device" : "Context";
 
         if (listNumPasses)
         {
