@@ -362,6 +362,7 @@ static void FreePreallocatedMemory()
             free(itr.second->pcSamplingData.pPcData[i].stallReason);
         }
         free(itr.second->pcSamplingData.pPcData);
+        std::vector<CUpti_PCSamplingConfigurationInfo>().swap(itr.second->pcSamplingConfigurationInfo);
 
         for (size_t i = 0; i < itr.second->pcSamplingStallReasons.numStallReasons; i++)
         {
@@ -381,6 +382,7 @@ static void FreePreallocatedMemory()
             free(itr->pcSamplingData.pPcData[i].stallReason);
         }
         free(itr->pcSamplingData.pPcData);
+        std::vector<CUpti_PCSamplingConfigurationInfo>().swap(itr->pcSamplingConfigurationInfo);
 
         for (size_t i = 0; i < itr->pcSamplingStallReasons.numStallReasons; i++)
         {
@@ -466,6 +468,7 @@ ConfigureActivity(
     }
 
     std::vector<CUpti_PCSamplingConfigurationInfo> pcSamplingConfigurationInfo;
+    pcSamplingConfigurationInfo.reserve(10);
 
     stallReason.attributeType = CUPTI_PC_SAMPLING_CONFIGURATION_ATTR_TYPE_STALL_REASON;
     stallReason.attributeData.stallReasonData.stallReasonCount = numStallReasons;
