@@ -258,7 +258,22 @@ void ProfilerCallbackHandler(
             }
         }
     }
+    else if (domain == CUPTI_CB_DOMAIN_STATE)
+    {
+        if (callbackId == CUPTI_CBID_STATE_FATAL_ERROR)
+        {
+            CUpti_StateData *pStateData = (CUpti_StateData*)pCallbackData;
+            const char *errorString = NULL;
+            CUPTI_API_CALL(cuptiGetResultString(pStateData->notification.result, &errorString));
 
+            std::cerr << "\nCUPTI encountered fatal error: " << errorString << std::endl;
+            std::cerr << "Error message: " << pStateData->notification.message << std::endl;
+
+            // Exiting the application if fatal error encountered in CUPTI
+            // If there is a CUPTI fatal error, it means CUPTI has stopped profiling the application.
+            exit(EXIT_FAILURE);
+        }
+    }
     return;
 }
 
@@ -294,6 +309,7 @@ void RegisterCallbacks()
     // Resource callback domain is needed for context creation callbacks
     CUPTI_API_CALL(cuptiEnableCallback(1, subscriber, CUPTI_CB_DOMAIN_RESOURCE, CUPTI_CBID_RESOURCE_CONTEXT_CREATED));
     CUPTI_API_CALL(cuptiEnableCallback(1, subscriber, CUPTI_CB_DOMAIN_RESOURCE, CUPTI_CBID_RESOURCE_CONTEXT_DESTROY_STARTING));
+    CUPTI_API_CALL(cuptiEnableDomain(1, subscriber, CUPTI_CB_DOMAIN_STATE));
 
     // Register callback for application exit
     atexit(EndExecution);
