@@ -227,6 +227,20 @@ template <>
   return __traits;
 };
 
+template <>
+[[nodiscard]] _CCCL_API constexpr arch_traits_t arch_traits<arch_id::sm_62>() noexcept
+{
+  auto __traits                                 = ::cuda::__common_arch_traits(arch_id::sm_62);
+  __traits.max_shared_memory_per_multiprocessor = 64 * 1024;
+  __traits.max_blocks_per_multiprocessor        = 32;
+  __traits.max_threads_per_multiprocessor       = 2048;
+  __traits.max_warps_per_multiprocessor         = __traits.max_threads_per_multiprocessor / __traits.warp_size;
+  __traits.max_shared_memory_per_block_optin    = 48 * 1024;
+  __traits.max_registers_per_block              = 32 * 1024;
+
+  return __traits;
+};
+
 template <>
 [[nodiscard]] _CCCL_API constexpr arch_traits_t arch_traits<arch_id::sm_70>() noexcept
 {
@@ -452,6 +466,8 @@ template <>
       return ::cuda::arch_traits<arch_id::sm_60>();
     case arch_id::sm_61:
       return ::cuda::arch_traits<arch_id::sm_61>();
+    case arch_id::sm_62:
+      return ::cuda::arch_traits<arch_id::sm_62>();
     case arch_id::sm_70:
       return ::cuda::arch_traits<arch_id::sm_70>();
     case arch_id::sm_75:
@@ -517,7 +533,7 @@ _LIBCUDACXX_BEGIN_NAMESPACE_CUDA_DEVICE
 //!
 //! @note This API cannot be used in constexpr context when compiling with nvc++ in CUDA mode.
 template <class _Dummy = void>
-[[nodiscard]] _CCCL_DEVICE_API _CCCL_TARGET_CONSTEXPR ::cuda::arch_traits_t current_arch_traits() noexcept
+[[nodiscard]] _CCCL_DEVICE_API inline _CCCL_TARGET_CONSTEXPR ::cuda::arch_traits_t current_arch_traits() noexcept
 {
 #    if _CCCL_DEVICE_COMPILATION()
   return ::cuda::arch_traits_for(::cuda::device::current_arch_id<_Dummy>());
