@@ -29,7 +29,15 @@
 #elif defined(_CCCL_IMPLICIT_SYSTEM_HEADER_MSVC)
 #  pragma system_header
 #endif // no system header
+
 #include <thrust/detail/execution_policy.h>
+#include <thrust/iterator/iterator_traits.h>
+#include <thrust/system/detail/adl/transform.h>
+#include <thrust/system/detail/generic/select_system.h>
+#include <thrust/system/detail/generic/transform.h>
+#include <thrust/transform.h>
+
+#include <cuda/std/iterator>
 
 THRUST_NAMESPACE_BEGIN
 
@@ -79,7 +87,7 @@ THRUST_NAMESPACE_BEGIN
 //!
 //! int data[10] = {-5, 0, 2, -3, 2, 4, 0, -1, 2, 8};
 //!
-//! thrust::negate<int> op;
+//! ::cuda::std::negate<int> op;
 //! thrust::transform(thrust::host, data, data + 10, data, op); // in-place transformation
 //!
 //! // data is now {5, 0, -2, 3, -2, -4, 0, 1, -2, -8};
@@ -91,13 +99,19 @@ THRUST_NAMESPACE_BEGIN
 //! @endrst
 //!
 //! \see https://en.cppreference.com/w/cpp/algorithm/transform
+_CCCL_EXEC_CHECK_DISABLE
 template <typename DerivedPolicy, typename InputIterator, typename OutputIterator, typename UnaryFunction>
 _CCCL_HOST_DEVICE OutputIterator transform(
   const thrust::detail::execution_policy_base<DerivedPolicy>& exec,
   InputIterator first,
   InputIterator last,
   OutputIterator result,
-  UnaryFunction op);
+  UnaryFunction op)
+{
+  _CCCL_NVTX_RANGE_SCOPE("thrust::transform");
+  using thrust::system::detail::generic::transform;
+  return transform(thrust::detail::derived_cast(thrust::detail::strip_const(exec)), first, last, result, op);
+}
 
 //! This version of \p transform applies a unary function to each element of an input sequence and stores the result in
 //! the corresponding position in an output sequence. Specifically, for each iterator <tt>i</tt> in the range [\p first,
@@ -130,7 +144,7 @@ _CCCL_HOST_DEVICE OutputIterator transform(
 //!
 //!  int data[10] = {-5, 0, 2, -3, 2, 4, 0, -1, 2, 8};
 //!
-//!  thrust::negate<int> op;
+//!  ::cuda::std::negate<int> op;
 //!  thrust::transform(data, data + 10, data, op); // in-place transformation
 //!
 //!  // data is now {5, 0, -2, 3, -2, -4, 0, 1, -2, -8};
@@ -143,7 +157,19 @@ _CCCL_HOST_DEVICE OutputIterator transform(
 //!
 //!  \see https://en.cppreference.com/w/cpp/algorithm/transform
 template <typename InputIterator, typename OutputIterator, typename UnaryFunction>
-OutputIterator transform(InputIterator first, InputIterator last, OutputIterator result, UnaryFunction op);
+OutputIterator transform(InputIterator first, InputIterator last, OutputIterator result, UnaryFunction op)
+{
+  _CCCL_NVTX_RANGE_SCOPE("thrust::transform");
+  using thrust::system::detail::generic::select_system;
+
+  using System1 = typename thrust::iterator_system<InputIterator>::type;
+  using System2 = typename thrust::iterator_system<OutputIterator>::type;
+
+  System1 system1;
+  System2 system2;
+
+  return thrust::transform(select_system(system1, system2), first, last, result, op);
+}
 
 //! This version of \p transform applies a binary function to each pair of elements from two input sequences and stores
 //! the result in the corresponding position in an output sequence. Specifically, for each iterator <tt>i</tt> in the
@@ -190,7 +216,7 @@ OutputIterator transform(InputIterator first, InputIterator last, OutputIterator
 //! int input2[6] = { 3,  6, -2,  1,  2,  3};
 //! int output[6];
 //!
-//! thrust::plus<int> op;
+//! ::cuda::std::plus<int> op;
 //! thrust::transform(thrust::host, input1, input1 + 6, input2, output, op);
 //!
 //! // output is now {-2,  6,  0,  4,  4,  7};
@@ -202,6 +228,7 @@ OutputIterator transform(InputIterator first, InputIterator last, OutputIterator
 //! @endrst
 //!
 //! \see https://en.cppreference.com/w/cpp/algorithm/transform
+_CCCL_EXEC_CHECK_DISABLE
 template <typename DerivedPolicy,
           typename InputIterator1,
           typename InputIterator2,
@@ -213,7 +240,12 @@ _CCCL_HOST_DEVICE OutputIterator transform(
   InputIterator1 last1,
   InputIterator2 first2,
   OutputIterator result,
-  BinaryFunction op);
+  BinaryFunction op)
+{
+  _CCCL_NVTX_RANGE_SCOPE("thrust::transform");
+  using thrust::system::detail::generic::transform;
+  return transform(thrust::detail::derived_cast(thrust::detail::strip_const(exec)), first1, last1, first2, result, op);
+}
 
 //! This version of \p transform applies a binary function to each pair of elements from two input sequences and stores
 //! the result in the corresponding position in an output sequence. Specifically, for each iterator <tt>i</tt> in the
@@ -253,7 +285,7 @@ _CCCL_HOST_DEVICE OutputIterator transform(
 //! int input2[6] = { 3,  6, -2,  1,  2,  3};
 //! int output[6];
 //!
-//! thrust::plus<int> op;
+//! ::cuda::std::plus<int> op;
 //! thrust::transform(input1, input1 + 6, input2, output, op);
 //!
 //! // output is now {-2,  6,  0,  4,  4,  7};
@@ -267,7 +299,21 @@ _CCCL_HOST_DEVICE OutputIterator transform(
 //! \see https://en.cppreference.com/w/cpp/algorithm/transform
 template <typename InputIterator1, typename InputIterator2, typename OutputIterator, typename BinaryFunction>
 OutputIterator
-transform(InputIterator1 first1, InputIterator1 last1, InputIterator2 first2, OutputIterator result, BinaryFunction op);
+transform(InputIterator1 first1, InputIterator1 last1, InputIterator2 first2, OutputIterator result, BinaryFunction op)
+{
+  _CCCL_NVTX_RANGE_SCOPE("thrust::transform");
+  using thrust::system::detail::generic::select_system;
+
+  using System1 = typename thrust::iterator_system<InputIterator1>::type;
+  using System2 = typename thrust::iterator_system<InputIterator2>::type;
+  using System3 = typename thrust::iterator_system<OutputIterator>::type;
+
+  System1 system1;
+  System2 system2;
+  System3 system3;
+
+  return thrust::transform(select_system(system1, system2, system3), first1, last1, first2, result, op);
+}
 
 /*! This version of \p transform_if conditionally applies a unary function
  *  to each element of an input sequence and stores the result in the corresponding
@@ -325,7 +371,7 @@ transform(InputIterator1 first1, InputIterator1 last1, InputIterator2 first2, Ou
  *    }
  *  };
  *
- *  thrust::negate<int> op;
+ *  ::cuda::std::negate<int> op;
  *
  *  // negate odd elements
  *  thrust::transform_if(thrust::host, data, data + 10, data, op, is_odd()); // in-place transformation
@@ -335,6 +381,7 @@ transform(InputIterator1 first1, InputIterator1 last1, InputIterator2 first2, Ou
  *
  *  \see thrust::transform
  */
+_CCCL_EXEC_CHECK_DISABLE
 template <typename DerivedPolicy,
           typename InputIterator,
           typename ForwardIterator,
@@ -346,7 +393,12 @@ _CCCL_HOST_DEVICE ForwardIterator transform_if(
   InputIterator last,
   ForwardIterator result,
   UnaryFunction op,
-  Predicate pred);
+  Predicate pred)
+{
+  _CCCL_NVTX_RANGE_SCOPE("thrust::transform_if");
+  using thrust::system::detail::generic::transform_if;
+  return transform_if(thrust::detail::derived_cast(thrust::detail::strip_const(exec)), first, last, result, op, pred);
+}
 
 /*! This version of \p transform_if conditionally applies a unary function
  *  to each element of an input sequence and stores the result in the corresponding
@@ -398,7 +450,7 @@ _CCCL_HOST_DEVICE ForwardIterator transform_if(
  *    }
  *  };
  *
- *  thrust::negate<int> op;
+ *  ::cuda::std::negate<int> op;
  *
  *  // negate odd elements
  *  thrust::transform_if(data, data + 10, data, op, is_odd()); // in-place transformation
@@ -410,7 +462,19 @@ _CCCL_HOST_DEVICE ForwardIterator transform_if(
  */
 template <typename InputIterator, typename ForwardIterator, typename UnaryFunction, typename Predicate>
 ForwardIterator
-transform_if(InputIterator first, InputIterator last, ForwardIterator result, UnaryFunction op, Predicate pred);
+transform_if(InputIterator first, InputIterator last, ForwardIterator result, UnaryFunction op, Predicate pred)
+{
+  _CCCL_NVTX_RANGE_SCOPE("thrust::transform_if");
+  using thrust::system::detail::generic::select_system;
+
+  using System1 = typename thrust::iterator_system<InputIterator>::type;
+  using System2 = typename thrust::iterator_system<ForwardIterator>::type;
+
+  System1 system1;
+  System2 system2;
+
+  return thrust::transform_if(select_system(system1, system2), first, last, result, op, pred);
+}
 
 /*! This version of \p transform_if conditionally applies a unary function
  *  to each element of an input sequence and stores the result in the corresponding
@@ -466,7 +530,7 @@ transform_if(InputIterator first, InputIterator last, ForwardIterator result, Un
  *  int data[10]    = {-5, 0, 2, -3, 2, 4, 0, -1, 2, 8};
  *  int stencil[10] = { 1, 0, 1,  0, 1, 0, 1,  0, 1, 0};
  *
- *  thrust::negate<int> op;
+ *  ::cuda::std::negate<int> op;
  *  ::cuda::std::identity identity;
  *
  *  thrust::transform_if(thrust::host, data, data + 10, stencil, data, op, identity); // in-place transformation
@@ -476,6 +540,7 @@ transform_if(InputIterator first, InputIterator last, ForwardIterator result, Un
  *
  *  \see thrust::transform
  */
+_CCCL_EXEC_CHECK_DISABLE
 template <typename DerivedPolicy,
           typename InputIterator1,
           typename InputIterator2,
@@ -489,7 +554,13 @@ _CCCL_HOST_DEVICE ForwardIterator transform_if(
   InputIterator2 stencil,
   ForwardIterator result,
   UnaryFunction op,
-  Predicate pred);
+  Predicate pred)
+{
+  _CCCL_NVTX_RANGE_SCOPE("thrust::transform_if");
+  using thrust::system::detail::generic::transform_if;
+  return transform_if(
+    thrust::detail::derived_cast(thrust::detail::strip_const(exec)), first, last, stencil, result, op, pred);
+}
 
 /*! This version of \p transform_if conditionally applies a unary function
  *  to each element of an input sequence and stores the result in the corresponding
@@ -538,7 +609,7 @@ _CCCL_HOST_DEVICE ForwardIterator transform_if(
  *  int data[10]    = {-5, 0, 2, -3, 2, 4, 0, -1, 2, 8};
  *  int stencil[10] = { 1, 0, 1,  0, 1, 0, 1,  0, 1, 0};
  *
- *  thrust::negate<int> op;
+ *  ::cuda::std::negate<int> op;
  *  ::cuda::std::identity identity;
  *
  *  thrust::transform_if(data, data + 10, stencil, data, op, identity); // in-place transformation
@@ -559,7 +630,21 @@ ForwardIterator transform_if(
   InputIterator2 stencil,
   ForwardIterator result,
   UnaryFunction op,
-  Predicate pred);
+  Predicate pred)
+{
+  _CCCL_NVTX_RANGE_SCOPE("thrust::transform_if");
+  using thrust::system::detail::generic::select_system;
+
+  using System1 = typename thrust::iterator_system<InputIterator1>::type;
+  using System2 = typename thrust::iterator_system<InputIterator2>::type;
+  using System3 = typename thrust::iterator_system<ForwardIterator>::type;
+
+  System1 system1;
+  System2 system2;
+  System3 system3;
+
+  return thrust::transform_if(select_system(system1, system2, system3), first, last, stencil, result, op, pred);
+}
 
 /*! This version of \p transform_if conditionally applies a binary function
  *  to each pair of elements from two input sequences and stores the result in the corresponding
@@ -620,7 +705,7 @@ ForwardIterator transform_if(
  *  int stencil[8] = { 1,  0,  1,  0,  1,  0};
  *  int output[6];
  *
- *  thrust::plus<int> op;
+ *  ::cuda::std::plus<int> op;
  *  ::cuda::std::identity identity;
  *
  *  thrust::transform_if(thrust::host, input1, input1 + 6, input2, stencil, output, op, identity);
@@ -630,6 +715,7 @@ ForwardIterator transform_if(
  *
  *  \see thrust::transform
  */
+_CCCL_EXEC_CHECK_DISABLE
 template <typename DerivedPolicy,
           typename InputIterator1,
           typename InputIterator2,
@@ -645,7 +731,20 @@ _CCCL_HOST_DEVICE ForwardIterator transform_if(
   InputIterator3 stencil,
   ForwardIterator result,
   BinaryFunction binary_op,
-  Predicate pred);
+  Predicate pred)
+{
+  _CCCL_NVTX_RANGE_SCOPE("thrust::transform_if");
+  using thrust::system::detail::generic::transform_if;
+  return transform_if(
+    thrust::detail::derived_cast(thrust::detail::strip_const(exec)),
+    first1,
+    last1,
+    first2,
+    stencil,
+    result,
+    binary_op,
+    pred);
+}
 
 /*! This version of \p transform_if conditionally applies a binary function
  *  to each pair of elements from two input sequences and stores the result in the corresponding
@@ -700,7 +799,7 @@ _CCCL_HOST_DEVICE ForwardIterator transform_if(
  *  int stencil[8] = { 1,  0,  1,  0,  1,  0};
  *  int output[6];
  *
- *  thrust::plus<int> op;
+ *  ::cuda::std::plus<int> op;
  *  ::cuda::std::identity identity;
  *
  *  thrust::transform_if(input1, input1 + 6, input2, stencil, output, op, identity);
@@ -723,11 +822,224 @@ ForwardIterator transform_if(
   InputIterator3 stencil,
   ForwardIterator result,
   BinaryFunction binary_op,
-  Predicate pred);
+  Predicate pred)
+{
+  _CCCL_NVTX_RANGE_SCOPE("thrust::transform_if");
+  using thrust::system::detail::generic::select_system;
+
+  using System1 = typename thrust::iterator_system<InputIterator1>::type;
+  using System2 = typename thrust::iterator_system<InputIterator2>::type;
+  using System3 = typename thrust::iterator_system<InputIterator3>::type;
+  using System4 = typename thrust::iterator_system<ForwardIterator>::type;
+
+  System1 system1;
+  System2 system2;
+  System3 system3;
+  System4 system4;
+
+  return thrust::transform_if(
+    select_system(system1, system2, system3, system4), first1, last1, first2, stencil, result, binary_op, pred);
+}
+
+//! Like \ref transform, but uses an element count instead of an iterator to the last element of the input sequence.
+template <typename DerivedPolicy, typename InputIterator, typename OutputIterator, typename UnaryFunction>
+_CCCL_HOST_DEVICE OutputIterator transform_n(
+  const detail::execution_policy_base<DerivedPolicy>& exec,
+  InputIterator first,
+  ::cuda::std::iter_difference_t<InputIterator> count,
+  OutputIterator result,
+  UnaryFunction op)
+{
+  _CCCL_NVTX_RANGE_SCOPE("thrust::transform_n");
+  using thrust::system::detail::generic::transform_n;
+  return transform_n(thrust::detail::derived_cast(thrust::detail::strip_const(exec)), first, count, result, op);
+}
+
+//! Like \ref transform, but uses an element count instead of an iterator to the last element of the input sequence.
+template <typename InputIterator, typename OutputIterator, typename UnaryFunction>
+OutputIterator transform_n(
+  InputIterator first, ::cuda::std::iter_difference_t<InputIterator> count, OutputIterator result, UnaryFunction op)
+{
+  _CCCL_NVTX_RANGE_SCOPE("thrust::transform_n");
+  iterator_system_t<InputIterator> system1;
+  iterator_system_t<OutputIterator> system2;
+  using thrust::system::detail::generic::select_system;
+  return thrust::transform_n(select_system(system1, system2), first, count, result, op);
+}
+
+//! Like \ref transform, but uses an element count instead of an iterator to the last element of the input sequence.
+template <typename DerivedPolicy,
+          typename InputIterator1,
+          typename InputIterator2,
+          typename OutputIterator,
+          typename BinaryFunction>
+_CCCL_HOST_DEVICE OutputIterator transform_n(
+  const detail::execution_policy_base<DerivedPolicy>& exec,
+  InputIterator1 first1,
+  ::cuda::std::iter_difference_t<InputIterator1> count,
+  InputIterator2 first2,
+  OutputIterator result,
+  BinaryFunction op)
+{
+  _CCCL_NVTX_RANGE_SCOPE("thrust::transform_n");
+  using thrust::system::detail::generic::transform_n;
+  return transform_n(thrust::detail::derived_cast(thrust::detail::strip_const(exec)), first1, count, first2, result, op);
+}
+
+//! Like \ref transform, but uses an element count instead of an iterator to the last element of the input sequence.
+template <typename InputIterator1, typename InputIterator2, typename OutputIterator, typename BinaryFunction>
+OutputIterator transform_n(
+  InputIterator1 first1,
+  ::cuda::std::iter_difference_t<InputIterator1> count,
+  InputIterator2 first2,
+  OutputIterator result,
+  BinaryFunction op)
+{
+  _CCCL_NVTX_RANGE_SCOPE("thrust::transform_n");
+  iterator_system_t<InputIterator1> system1;
+  iterator_system_t<InputIterator2> system2;
+  iterator_system_t<OutputIterator> system3;
+  using thrust::system::detail::generic::select_system;
+  return thrust::transform_n(select_system(system1, system2, system3), first1, count, first2, result, op);
+}
+
+//! Like \ref transform_if, but uses an element count instead of an iterator to the last element of the input sequence.
+template <typename DerivedPolicy,
+          typename InputIterator,
+          typename ForwardIterator,
+          typename UnaryFunction,
+          typename Predicate>
+_CCCL_HOST_DEVICE ForwardIterator transform_if_n(
+  const detail::execution_policy_base<DerivedPolicy>& exec,
+  InputIterator first,
+  ::cuda::std::iter_difference_t<InputIterator> count,
+  ForwardIterator result,
+  UnaryFunction op,
+  Predicate pred)
+{
+  _CCCL_NVTX_RANGE_SCOPE("thrust::transform_if_n");
+  using thrust::system::detail::generic::transform_if_n;
+  return transform_if_n(thrust::detail::derived_cast(thrust::detail::strip_const(exec)), first, count, result, op, pred);
+}
+
+//! Like \ref transform_if, but uses an element count instead of an iterator to the last element of the input sequence.
+template <typename InputIterator, typename ForwardIterator, typename UnaryFunction, typename Predicate>
+ForwardIterator transform_if_n(
+  InputIterator first,
+  ::cuda::std::iter_difference_t<InputIterator> count,
+  ForwardIterator result,
+  UnaryFunction op,
+  Predicate pred)
+{
+  _CCCL_NVTX_RANGE_SCOPE("thrust::transform_if_n");
+  iterator_system_t<InputIterator> system1;
+  iterator_system_t<ForwardIterator> system2;
+  using thrust::system::detail::generic::select_system;
+  return thrust::transform_if_n(select_system(system1, system2), first, count, result, op, pred);
+}
+
+//! Like \ref transform_if, but uses an element count instead of an iterator to the last element of the input sequence.
+template <typename DerivedPolicy,
+          typename InputIterator1,
+          typename InputIterator2,
+          typename ForwardIterator,
+          typename UnaryFunction,
+          typename Predicate>
+_CCCL_HOST_DEVICE ForwardIterator transform_if_n(
+  const detail::execution_policy_base<DerivedPolicy>& exec,
+  InputIterator1 first,
+  ::cuda::std::iter_difference_t<InputIterator1> count,
+  InputIterator2 stencil,
+  ForwardIterator result,
+  UnaryFunction op,
+  Predicate pred)
+{
+  _CCCL_NVTX_RANGE_SCOPE("thrust::transform_if_n");
+  using thrust::system::detail::generic::transform_if_n;
+  return transform_if_n(
+    thrust::detail::derived_cast(thrust::detail::strip_const(exec)), first, count, stencil, result, op, pred);
+}
+
+//! Like \ref transform_if, but uses an element count instead of an iterator to the last element of the input sequence.
+template <typename InputIterator1,
+          typename InputIterator2,
+          typename ForwardIterator,
+          typename UnaryFunction,
+          typename Predicate>
+ForwardIterator transform_if_n(
+  InputIterator1 first,
+  ::cuda::std::iter_difference_t<InputIterator1> count,
+  InputIterator2 stencil,
+  ForwardIterator result,
+  UnaryFunction op,
+  Predicate pred)
+{
+  _CCCL_NVTX_RANGE_SCOPE("thrust::transform_if_n");
+  iterator_system_t<InputIterator1> system1;
+  iterator_system_t<InputIterator2> system2;
+  iterator_system_t<ForwardIterator> system3;
+  using thrust::system::detail::generic::select_system;
+  return thrust::transform_if_n(select_system(system1, system2, system3), first, count, stencil, result, op, pred);
+}
+
+//! Like \ref transform_if, but uses an element count instead of an iterator to the last element of the input sequence.
+template <typename DerivedPolicy,
+          typename InputIterator1,
+          typename InputIterator2,
+          typename InputIterator3,
+          typename ForwardIterator,
+          typename BinaryFunction,
+          typename Predicate>
+_CCCL_HOST_DEVICE ForwardIterator transform_if_n(
+  const detail::execution_policy_base<DerivedPolicy>& exec,
+  InputIterator1 first1,
+  ::cuda::std::iter_difference_t<InputIterator1> count,
+  InputIterator2 first2,
+  InputIterator3 stencil,
+  ForwardIterator result,
+  BinaryFunction binary_op,
+  Predicate pred)
+{
+  _CCCL_NVTX_RANGE_SCOPE("thrust::transform_if_n");
+  using thrust::system::detail::generic::transform_if_n;
+  return transform_if_n(
+    thrust::detail::derived_cast(thrust::detail::strip_const(exec)),
+    first1,
+    count,
+    first2,
+    stencil,
+    result,
+    binary_op,
+    pred);
+}
+
+//! Like \ref transform_if, but uses an element count instead of an iterator to the last element of the input sequence.
+template <typename InputIterator1,
+          typename InputIterator2,
+          typename InputIterator3,
+          typename ForwardIterator,
+          typename BinaryFunction,
+          typename Predicate>
+ForwardIterator transform_if_n(
+  InputIterator1 first1,
+  ::cuda::std::iter_difference_t<InputIterator1> count,
+  InputIterator2 first2,
+  InputIterator3 stencil,
+  ForwardIterator result,
+  BinaryFunction binary_op,
+  Predicate pred)
+{
+  _CCCL_NVTX_RANGE_SCOPE("thrust::transform_if_n");
+  iterator_system_t<InputIterator1> system1;
+  iterator_system_t<InputIterator2> system2;
+  iterator_system_t<InputIterator3> system3;
+  iterator_system_t<ForwardIterator> system4;
+  using thrust::system::detail::generic::select_system;
+  return thrust::transform_if_n(
+    select_system(system1, system2, system3, system4), first1, count, first2, stencil, result, binary_op, pred);
+}
 
 /*! \} // end transformations
  */
 
 THRUST_NAMESPACE_END
-
-#include <thrust/detail/transform.inl>
