@@ -1,5 +1,5 @@
 /*
- * Copyright 2024 NVIDIA Corporation.  All rights reserved.
+ * Copyright 2024-2025 NVIDIA Corporation.  All rights reserved.
  *
  * NOTICE TO LICENSEE:
  *
@@ -275,8 +275,25 @@ __nv_cvt_halfraw_to_fp4(const __half_raw x,
                         const enum cudaRoundMode rounding) {
     assert((rounding == cudaRoundNearest) || (rounding == cudaRoundZero));
     __nv_fp4_storage_t res = 0U;
-    float fx = __internal_halfraw_to_float(x);
-    res = __nv_cvt_float_to_fp4(fx, fp4_interpretation, rounding);
+
+#if (__CUDA_FP8_INTERNAL_CAN_RELY_ON_PTX_FOR_SHORTTYPESCVT__)
+    if (rounding == cudaRoundNearest)
+    {
+        unsigned int half2_storage = (unsigned int)(x.x);
+        unsigned short tmp;
+        asm("{ .reg .b8 __$temp1;                           \n"
+            " cvt.rn.satfinite.e2m1x2.f16x2 __$temp1, %1;   \n"
+            " mov.b16 %0, {__$temp1, 0};                   }\n"
+            : "=h"(tmp)
+            : "r"(half2_storage));
+        res = (__nv_fp4_storage_t)(tmp);
+    } else
+#endif
+    {
+        float fx = __internal_halfraw_to_float(x);
+        res = __nv_cvt_float_to_fp4(fx, fp4_interpretation, rounding);
+    }
+
     return res;
 }
 
@@ -286,16 +303,31 @@ __CUDA_HOSTDEVICE_FP4_DECL__ __nv_fp4x2_storage_t __nv_cvt_halfraw2_to_fp4x2(
     const enum cudaRoundMode rounding) {
     assert((rounding == cudaRoundNearest) || (rounding == cudaRoundZero));
     unsigned short tmp;
-    __half_raw raw;
-    raw.x = x.x;
-    __nv_fp4_storage_t lo =
-        __nv_cvt_halfraw_to_fp4(raw, fp4_interpretation, rounding);
-    raw.x = x.y;
-    __nv_fp4_storage_t hi =
-        __nv_cvt_halfraw_to_fp4(raw, fp4_interpretation, rounding);
-    tmp = hi;
-    tmp = (__nv_fp4x2_storage_t)(tmp << 4U);
-    tmp = (__nv_fp4x2_storage_t)(tmp | lo);
+
+#if (__CUDA_FP8_INTERNAL_CAN_RELY_ON_PTX_FOR_SHORTTYPESCVT__)
+    if (rounding == cudaRoundNearest)
+    {
+        unsigned int half2_storage;
+        (void)memcpy(&half2_storage, &x, sizeof(x));
+        asm("{ .reg .b8 __$temp1;                           \n"
+            " cvt.rn.satfinite.e2m1x2.f16x2 __$temp1, %1;   \n"
+            " mov.b16 %0, {__$temp1, 0};                   }\n"
+            : "=h"(tmp)
+            : "r"(half2_storage));
+    } else
+#endif
+    {
+        __half_raw raw;
+        raw.x = x.x;
+        __nv_fp4_storage_t lo =
+            __nv_cvt_halfraw_to_fp4(raw, fp4_interpretation, rounding);
+        raw.x = x.y;
+        __nv_fp4_storage_t hi =
+            __nv_cvt_halfraw_to_fp4(raw, fp4_interpretation, rounding);
+        tmp = hi;
+        tmp = (__nv_fp4x2_storage_t)(tmp << 4U);
+        tmp = (__nv_fp4x2_storage_t)(tmp | lo);
+    }
     return (__nv_fp4x2_storage_t)tmp;
 }
 
@@ -303,9 +335,26 @@ __CUDA_HOSTDEVICE_FP4_DECL__ __nv_fp4_storage_t __nv_cvt_bfloat16raw_to_fp4(
     const __nv_bfloat16_raw x,
     const __nv_fp4_interpretation_t fp4_interpretation,
     const enum cudaRoundMode rounding) {
-    const float fx = __internal_bf16raw_to_float(x);
-    const __nv_fp4_storage_t res =
-        __nv_cvt_float_to_fp4(fx, fp4_interpretation, rounding);
+
+    __nv_fp4_storage_t res = 0U;
+#if (__CUDA_FP8_INTERNAL_CAN_RELY_ON_PTX_FOR_SHORTTYPESCVT__)
+    if (rounding == cudaRoundNearest)
+    {
+        unsigned int bf162_storage = (unsigned int)(x.x);
+        unsigned short tmp;
+        asm("{ .reg .b8 __$temp1;                           \n"
+            " cvt.rn.satfinite.e2m1x2.bf16x2 __$temp1, %1;  \n"
+            " mov.b16 %0, {__$temp1, 0};                   }\n"
+            : "=h"(tmp)
+            : "r"(bf162_storage));
+        res = (__nv_fp4_storage_t)(tmp);
+    } else
+#endif
+    {
+        float fx = __internal_bf16raw_to_float(x);
+        res = __nv_cvt_float_to_fp4(fx, fp4_interpretation, rounding);
+    }
+
     return res;
 }
 
@@ -314,17 +363,34 @@ __nv_cvt_bfloat16raw2_to_fp4x2(
     const __nv_bfloat162_raw x,
     const __nv_fp4_interpretation_t fp4_interpretation,
     const enum cudaRoundMode rounding) {
-    __nv_bfloat16_raw raw;
-    raw.x = x.y;
-    __nv_fp4x2_storage_t storage =
-        (__nv_fp4x2_storage_t)__nv_cvt_bfloat16raw_to_fp4(raw,
-                                        fp4_interpretation, rounding);
-    storage = (__nv_fp4x2_storage_t)(storage << 4U);
-    raw.x = x.x;
-    storage = (__nv_fp4x2_storage_t)(storage |
-                                     __nv_cvt_bfloat16raw_to_fp4(raw,
-                                        fp4_interpretation, rounding));
-    return storage;
+
+    unsigned short tmp;
+
+#if (__CUDA_FP8_INTERNAL_CAN_RELY_ON_PTX_FOR_SHORTTYPESCVT__)
+    if (rounding == cudaRoundNearest)
+    {
+        unsigned int bf162_storage;
+        (void)memcpy(&bf162_storage, &x, sizeof(x));
+        asm("{ .reg .b8 __$temp1;                           \n"
+            " cvt.rn.satfinite.e2m1x2.bf16x2 __$temp1, %1;  \n"
+            " mov.b16 %0, {__$temp1, 0};                   }\n"
+            : "=h"(tmp)
+            : "r"(bf162_storage));
+    } else
+#endif
+    {
+        __nv_bfloat16_raw raw;
+        raw.x = x.x;
+        __nv_fp4_storage_t lo =
+            __nv_cvt_bfloat16raw_to_fp4(raw, fp4_interpretation, rounding);
+        raw.x = x.y;
+        __nv_fp4_storage_t hi =
+            __nv_cvt_bfloat16raw_to_fp4(raw, fp4_interpretation, rounding);
+        tmp = hi;
+        tmp = (__nv_fp4x2_storage_t)(tmp << 4U);
+        tmp = (__nv_fp4x2_storage_t)(tmp | lo);
+    }
+    return (__nv_fp4x2_storage_t)tmp;
 }
 
 __CUDA_HOSTDEVICE_FP4_DECL__ __half2_raw
