@@ -54,7 +54,7 @@
  * This file contains all the C APIs to perform GPUDirect Storage supported IO operations
  */
 
-#ifdef __cplusplus
+#if __cplusplus
 extern "C"
 {
 #endif
@@ -206,8 +206,10 @@ typedef enum CUfileDriverStatusFlags {
     CU_FILE_NVMESH_SUPPORTED = 8, /*!< Support for NVMesh Block Dev*/
     CU_FILE_BEEGFS_SUPPORTED = 9, /*!< Support for BeeGFS */
     //10 is reserved for YRCloudFile
-    CU_FILE_NVME_P2P_SUPPORTED = 11,   /*!< Support for NVMe using PCI P2PDMA */
+    CU_FILE_NVME_P2P_SUPPORTED = 11,   /*!< Do not use this macro. This is deprecated now */
     CU_FILE_SCATEFS_SUPPORTED = 12,      /*!< Support for ScateFS */
+    CU_FILE_VIRTIOFS_SUPPORTED = 13,   /*!<Support for VirtioFS */
+    CU_FILE_MAX_TARGET_TYPES,       /*!<Maximum FS supported */
 }CUfileDriverStatusFlags_t;
 
 typedef enum CUfileDriverControlFlags {
@@ -220,13 +222,34 @@ typedef enum CUfileDriverControlFlags {
 typedef enum CUfileFeatureFlags {
         CU_FILE_DYN_ROUTING_SUPPORTED = 0, /*!< Support for Dynamic routing to handle devices across the PCIe bridges */
 
-        CU_FILE_BATCH_IO_SUPPORTED = 1, /*!<  Unsupported */
+        CU_FILE_BATCH_IO_SUPPORTED = 1, /*!<  Supported */
 
-        CU_FILE_STREAMS_SUPPORTED = 2, /*!<  Unsupported */
+        CU_FILE_STREAMS_SUPPORTED = 2, /*!<  Supported */
 
-        CU_FILE_PARALLEL_IO_SUPPORTED = 3 /*!<  Unsupported */
+        CU_FILE_PARALLEL_IO_SUPPORTED = 3, /*!<  Supported */
+
+        CU_FILE_P2P_SUPPORTED = 4 /*!< Support for PCI P2PDMA */
 }CUfileFeatureFlags_t;
 
+typedef enum CUfileP2PFlags {
+        CUFILE_P2PDMA = 0, /*!< Support for PCI P2PDMA */
+        CUFILE_NVFS = 1,   /*!< Support for nvidia-fs */
+        CUFILE_DMABUF = 2, /*!< Support for DMA Buffer */
+        CUFILE_C2C = 3,     /*!< Support for Chip-to-Chip (Grace-based systems) */
+        CUFILE_NVIDIA_PEERMEM = 4 /*!< Only for IBM Spectrum Scale and WekaFS */
+}CUfileP2PFlags_t;
+
+/* P2P Flag constants for use with cuFileDriverSetP2PFlags */
+#define CU_FILE_P2P_FLAG_PCI_P2PDMA ((CUfileP2PFlags_t)(1 << CUFILE_P2PDMA))
+#define CU_FILE_P2P_FLAG_NVFS ((CUfileP2PFlags_t)(1 << CUFILE_NVFS))
+#define CU_FILE_P2P_FLAG_DMABUF ((CUfileP2PFlags_t)(1 << CUFILE_DMABUF))
+#define CU_FILE_P2P_FLAG_C2C ((CUfileP2PFlags_t)(1 << CUFILE_C2C))
+
+typedef enum CUfileOpcode {
+		CUFILE_READ = 0,
+		CUFILE_WRITE
+}CUfileOpcode_t;
+
 typedef struct CUfileDrvProps {
         struct {
                 unsigned int major_version;
@@ -271,18 +294,18 @@ typedef struct cufileRDMAInfo
 
 typedef struct CUfileFSOps {
       /* NULL means discover using fstat */
-      const char* (*fs_type) (void *handle);
+      const char* (*fs_type) (const void *handle);
 
       /* list of host addresses to use,  NULL means no restriction */
-      int (*getRDMADeviceList)(void *handle, sockaddr_t **hostaddrs);
+      int (*getRDMADeviceList)(const void *handle, sockaddr_t **hostaddrs);
 
       /* -1 no pref */
-      int (*getRDMADevicePriority)(void *handle, char*, size_t,
-                                loff_t, sockaddr_t* hostaddr);
+      int (*getRDMADevicePriority)(const void *handle, char*, size_t,
+                                loff_t, const sockaddr_t* hostaddr);
 
       /* NULL means try VFS */
-      ssize_t (*read) (void *handle, char*, size_t, loff_t, cufileRDMAInfo_t*);
-      ssize_t (*write) (void *handle, const char *, size_t, loff_t , cufileRDMAInfo_t*);
+      ssize_t (*read) (const void *handle, char*, size_t, loff_t, const cufileRDMAInfo_t*);
+      ssize_t (*write) (const void *handle, const char *, size_t, loff_t , const cufileRDMAInfo_t*);
 }CUfileFSOps_t;
 
 /* File Handle */
@@ -553,10 +576,6 @@ CUfileError_t cuFileDriverSetMaxPinnedMemSize(size_t max_pinned_size);
 //Experimental Batch API's
 
 
-typedef enum CUfileOpcode {
-		CUFILE_READ = 0,
-		CUFILE_WRITE
-}CUfileOpcode_t;
 
 typedef enum CUFILEStatus_enum {
 	CUFILE_WAITING = 0x000001,  /* required value prior to submission */
@@ -762,7 +781,7 @@ typedef enum CUFileBoolConfigParameter_t {
 	CUFILE_PARAM_FORCE_COMPAT_MODE,
 	CUFILE_PARAM_FS_MISC_API_CHECK_AGGRESSIVE,
 	CUFILE_PARAM_EXECUTION_PARALLEL_IO,
-	CUFILE_PARAM_PROFILE_NVTX,
+	CUFILE_PARAM_PROFILE_NVTX, //Do not use this macro. This is deprecated now.
 	CUFILE_PARAM_PROPERTIES_ALLOW_SYSTEM_MEMORY,
 	CUFILE_PARAM_USE_PCIP2PDMA,
 	CUFILE_PARAM_PREFER_IO_URING,
@@ -1103,6 +1122,84 @@ CUfileError_t cuFileGetParameterPosixPoolSlabArray(
                                      size_t *count_values,
                                      int len);
 
+/**
+ * @brief
+ * Gets the P2P flags for a specific filesystem or block device
+ *
+ * @param  status_flag The filesystem/device status flag (e.g., CU_FILE_LUSTRE_SUPPORTED)
+ * @param  p2p_flags Pointer to store the P2P flags
+ *
+ * @return CU_FILE_SUCCESS on success
+ * @return CU_FILE_DRIVER_NOT_INITIALIZED if the driver is not initialized
+ * @return CU_FILE_INVALID_VALUE if p2p_flags is NULL
+ *
+ * @see cuFileDriverSetP2PFlags
+ * @see cuFileDriverGetProperties
+ */
+CUfileError_t cuFileDriverGetP2PFlags(CUfileDriverStatusFlags_t status_flag, CUfileP2PFlags_t *p2p_flags);
+
+/**
+ * @brief
+ * Sets the P2P flags for a specific filesystem or block device
+ *
+ * This function configures the P2P and NVFS capabilities for a given filesystem
+ * or block device type. The flags parameter should be a bitmask of the desired
+ * P2P capabilities.
+ *
+ * @param[in] status_flag The filesystem/device status flag (e.g., CU_FILE_LUSTRE_SUPPORTED,
+ *                        CU_FILE_NVME_SUPPORTED, etc.)
+ * @param[in] p2p_flags   The P2P flags bitmask to set
+ *
+ * @return CU_FILE_SUCCESS on success
+ * @return CU_FILE_DRIVER_NOT_INITIALIZED if the cuFile driver is not initialized
+ * @return CU_FILE_INVALID_VALUE if status_flag is invalid
+ * @return CU_FILE_P2P_FLAG_NOT_SUPPORTED if p2p_flags contains flags not supported
+ *                               on the current platform
+ *
+ * @par Platform-Specific Restrictions:
+ * This API enforces strict platform-specific flag restrictions:
+ * - **x86_64 platforms**: Only CU_FILE_P2P_FLAG_PCI_P2PDMA is allowed
+ * - **AARCH64 platforms**: Only CU_FILE_P2P_FLAG_C2C is allowed
+ * - **Cross-platform flags**: CU_FILE_P2P_FLAG_NVFS and CU_FILE_P2P_FLAG_DMABUF are NOT allowed
+ *                            to be set via this API (they are managed internally by the driver)
+ *
+ * @par Usage Example:
+ * @code
+ * // x86_64 platform - only P2PDMA allowed
+ * #ifndef AARCH64_PLATFORM
+ *     CUfileP2PFlags_t flags = CU_FILE_P2P_FLAG_PCI_P2PDMA;
+ *     CUfileError_t error = cuFileDriverSetP2PFlags(CU_FILE_NVME_SUPPORTED, flags);
+ *     if (error.err != CU_FILE_SUCCESS) {
+ *         fprintf(stderr, "Failed to set P2P flags: %d\n", error.err);
+ *     }
+ * #endif
+ * 
+ * // AARCH64 platform - only C2C allowed
+ * #ifdef AARCH64_PLATFORM
+ *     CUfileP2PFlags_t flags = CU_FILE_P2P_FLAG_C2C;
+ *     CUfileError_t error = cuFileDriverSetP2PFlags(CU_FILE_NVME_SUPPORTED, flags);
+ *     if (error.err != CU_FILE_SUCCESS) {
+ *         fprintf(stderr, "Failed to set P2P flags: %d\n", error.err);
+ *     }
+ * #endif
+ * 
+ * // This will fail on any platform (NVFS not allowed via API)
+ * CUfileP2PFlags_t invalid_flags = CU_FILE_P2P_FLAG_NVFS;
+ * error = cuFileDriverSetP2PFlags(CU_FILE_NVME_SUPPORTED, invalid_flags);
+ * // error.err will be CU_FILE_P2P_FLAG_NOT_SUPPORTED
+ * @endcode
+ *
+ * @warning This API only allows platform-specific P2P flags (P2PDMA on x86_64, C2C on AARCH64)
+ * @warning NVFS and DMABUF flags are managed internally by the driver and cannot be set via this API
+ * @warning Setting unsupported flags will result in CU_FILE_P2P_FLAG_NOT_SUPPORTED error
+ *
+ * @note The function validates that only platform-appropriate flags are provided
+ * @note Use cuFileDriverGetP2PFlags() to query current flag status including NVFS and DMABUF
+ *
+ * @see cuFileDriverGetP2PFlags
+ */
+CUfileError_t cuFileDriverSetP2PFlags(CUfileDriverStatusFlags_t status_flag, CUfileP2PFlags_t p2p_flags);
+
 #pragma GCC visibility pop
 
 /// @cond DOXYGEN_SKIP_MACRO
