@@ -57,7 +57,6 @@ This file contains the CUPTI profiling host API's.
 #include <cupti_result.h>
 #include <stdint.h>
 #include <stddef.h>
-#include <string>
 
 #ifdef __cplusplus
 extern "C" {
@@ -84,6 +83,13 @@ typedef enum CUpti_MetricType
     CUPTI_METRIC_TYPE__COUNT
 } CUpti_MetricType;
 
+typedef enum CUpti_MetricCollectionScope
+{
+    CUPTI_METRIC_COLLECTION_SCOPE_CONTEXT = 0,
+    CUPTI_METRIC_COLLECTION_SCOPE_DEVICE,
+    CUPTI_METRIC_COLLECTION_SCOPE_INVALID
+} CUpti_MetricCollectionScope;
+
 typedef enum CUpti_ProfilerType
 {
     CUPTI_PROFILER_TYPE_RANGE_PROFILER,
@@ -271,9 +277,11 @@ typedef struct CUpti_Profiler_Host_GetMetricProperties_Params
     const char* pDimUnit;
     /// [out] the metric type (counter, ratio or throughput)
     CUpti_MetricType metricType;
+    /// [out] the metric collection scope (context, device)
+    CUpti_MetricCollectionScope metricCollectionScope;
 } CUpti_Profiler_Host_GetMetricProperties_Params;
 
-#define CUpti_Profiler_Host_GetMetricProperties_Params_STRUCT_SIZE                  CUPTI_PROFILER_STRUCT_SIZE(CUpti_Profiler_Host_GetMetricProperties_Params, metricType)
+#define CUpti_Profiler_Host_GetMetricProperties_Params_STRUCT_SIZE                  CUPTI_PROFILER_STRUCT_SIZE(CUpti_Profiler_Host_GetMetricProperties_Params, metricCollectionScope)
 
 /**
  * \brief Get the properties of the metric.
@@ -528,7 +536,7 @@ typedef struct CUpti_Profiler_Host_GetMaxNumHardwareMetricsPerPass_Params
  */
 CUptiResult CUPTIAPI cuptiProfilerHostGetMaxNumHardwareMetricsPerPass(CUpti_Profiler_Host_GetMaxNumHardwareMetricsPerPass_Params* pParams);
 
-/** @} */ /* END CUPTI_METRIC_API */
+/** @} */ /* END CUPTI_PROFILER_HOST_API */
 #if defined(__GNUC__) && defined(CUPTI_LIB)
     #pragma GCC visibility pop
 #endif
