@@ -557,7 +557,7 @@ typedef struct CUpti_Profiler_GetCounterAvailability_Params
     uint8_t* pCounterAvailabilityImage;                 //!< [in] buffer receiving counter availability image, may be NULL
     bool bAllowDeviceLevelCounters;                      //!< [in] if true, device level counters are included in the counter availability image
 } CUpti_Profiler_GetCounterAvailability_Params;
-#define CUpti_Profiler_GetCounterAvailability_Params_STRUCT_SIZE                  CUPTI_PROFILER_STRUCT_SIZE(CUpti_Profiler_GetCounterAvailability_Params, pCounterAvailabilityImage)
+#define CUpti_Profiler_GetCounterAvailability_Params_STRUCT_SIZE                  CUPTI_PROFILER_STRUCT_SIZE(CUpti_Profiler_GetCounterAvailability_Params, bAllowDeviceLevelCounters)
 
 /**
  * \brief Query counter availibility
