@@ -533,7 +533,34 @@ typedef enum CUpti_runtime_api_trace_cbid_enum {
     CUPTI_RUNTIME_TRACE_CBID_cudaMemSetMemPool_v13000                                      = 520,
     CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetHostAtomicCapabilities_v13000                    = 521,
     CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetP2PAtomicCapabilities_v13000                     = 522,
-    CUPTI_RUNTIME_TRACE_CBID_SIZE                                                          = 523,
+    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetDevResource_v13010                               = 523,
+    CUPTI_RUNTIME_TRACE_CBID_cudaDevSmResourceSplitByCount_v13010                          = 524,
+    CUPTI_RUNTIME_TRACE_CBID_cudaDevResourceGenerateDesc_v13010                            = 525,
+    CUPTI_RUNTIME_TRACE_CBID_cudaGreenCtxCreate_v13010                                     = 526,
+    CUPTI_RUNTIME_TRACE_CBID_cudaExecutionCtxDestroy_v13010                                = 527,
+    CUPTI_RUNTIME_TRACE_CBID_cudaExecutionCtxGetDevResource_v13010                         = 528,
+    CUPTI_RUNTIME_TRACE_CBID_cudaExecutionCtxGetDevice_v13010                              = 529,
+    CUPTI_RUNTIME_TRACE_CBID_cudaExecutionCtxGetId_v13010                                  = 530,
+    CUPTI_RUNTIME_TRACE_CBID_cuda531_v13010                                                = 531,
+    CUPTI_RUNTIME_TRACE_CBID_cuda532_v13010                                                = 532,
+    CUPTI_RUNTIME_TRACE_CBID_cudaExecutionCtxStreamCreate_v13010                           = 533,
+    CUPTI_RUNTIME_TRACE_CBID_cudaExecutionCtxSynchronize_v13010                            = 534,
+    CUPTI_RUNTIME_TRACE_CBID_cudaGraphConditionalHandleCreate_v2_v13010                    = 535,
+    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetDevResource_v13010                               = 536,
+    CUPTI_RUNTIME_TRACE_CBID_cudaStreamGetDevResource_ptsz_v13010                          = 537,
+    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetContainingGraph_v13010                        = 538,
+    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetLocalId_v13010                                = 539,
+    CUPTI_RUNTIME_TRACE_CBID_cudaGraphNodeGetToolsId_v13010                                = 540,
+    CUPTI_RUNTIME_TRACE_CBID_cudaGraphGetId_v13010                                         = 541,
+    CUPTI_RUNTIME_TRACE_CBID_cudaGraphExecGetId_v13010                                     = 542,
+    CUPTI_RUNTIME_TRACE_CBID_cudaExecutionCtxRecordEvent_v13010                            = 543,
+    CUPTI_RUNTIME_TRACE_CBID_cudaExecutionCtxWaitEvent_v13010                              = 544,
+    CUPTI_RUNTIME_TRACE_CBID_cuda545_v13010                                                = 545,
+    CUPTI_RUNTIME_TRACE_CBID_cuda546_v13010                                                = 546,
+    CUPTI_RUNTIME_TRACE_CBID_cuda547_v13010                                                = 547,
+    CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetExecutionCtx_v13010                              = 548,
+    CUPTI_RUNTIME_TRACE_CBID_cudaDevSmResourceSplit_v13010                                 = 549,
+    CUPTI_RUNTIME_TRACE_CBID_SIZE                                                          = 550,
     CUPTI_RUNTIME_TRACE_CBID_FORCE_INT                                                     = 0x7fffffff
 } CUpti_runtime_api_trace_cbid;
 
