@@ -1,7 +1,8 @@
- /* Copyright 2009-2023 NVIDIA CORPORATION & AFFILIATES.  All rights reserved.
+ /* Copyright 2009-2025 NVIDIA CORPORATION & AFFILIATES.  All rights reserved.
   * 
   * NOTICE TO LICENSEE: 
   * 
+  * 
   * The source code and/or documentation ("Licensed Deliverables") are 
   * subject to NVIDIA intellectual property rights under U.S. and 
   * international Copyright laws. 
@@ -195,6 +196,7 @@ typedef enum /**< NppStatus NPP return value */
 {
     /* negative return-codes indicate errors */
     NPP_NOT_SUPPORTED_MODE_ERROR            = -9999,    /**< Not supported mode error.*/
+    NPP_LIBRARY_VERSION_MISMATCH_ERROR      = -9998,    /**< Application and NPP library version mismatch.*/
     
     NPP_INVALID_HOST_POINTER_ERROR          = -1032,    /**< Invalid host memory pointer error.*/
     NPP_INVALID_DEVICE_POINTER_ERROR        = -1031,    /**< Invalid device memory pointer error.*/
@@ -272,7 +274,7 @@ typedef enum /**< NppStatus NPP return value */
     NPP_NO_OPERATION_WARNING                = 1,        /**<  Indicates that no operation was performed */
     NPP_DIVIDE_BY_ZERO_WARNING              = 6,        /**<  Divisor is zero however does not terminate the execution */
     NPP_AFFINE_QUAD_INCORRECT_WARNING       = 28,       /**<  Indicates that the quadrangle passed to one of affine warping functions doesn't have necessary properties. First 3 vertices are used, the fourth vertex discarded. */
-    NPP_WRONG_INTERSECTION_ROI_WARNING      = 29,       /**<  The given ROI has no interestion with either the source or destination ROI. Thus no operation was performed. */
+    NPP_WRONG_INTERSECTION_ROI_WARNING      = 29,       /**<  The given ROI has no intersection with either the source or destination ROI. Thus no operation was performed. */
     NPP_WRONG_INTERSECTION_QUAD_WARNING     = 30,       /**<  The given quadrangle has no intersection with either the source or destination ROI. Thus no operation was performed. */
     NPP_DOUBLE_SIZE_WARNING                 = 35,       /**<  Image size isn't multiple of two. Indicates that in case of 422/411/420 sampling the ROI width/height was modified for proper processing. */
     
@@ -903,7 +905,7 @@ typedef enum
     NPP_WATERSHED_SEGMENT_BOUNDARIES_NONE,      /**< Image watershed segment boundary type none. */
     NPP_WATERSHED_SEGMENT_BOUNDARIES_BLACK,     /**< Image watershed segment boundary type black. */
     NPP_WATERSHED_SEGMENT_BOUNDARIES_WHITE,     /**< Image watershed segment boundary type white. */
-    NPP_WATERSHED_SEGMENT_BOUNDARIES_CONTRAST,  /**< Image watershed segment boundary type contrasting intensiity. */
+    NPP_WATERSHED_SEGMENT_BOUNDARIES_CONTRAST,  /**< Image watershed segment boundary type contrasting intensity. */
     NPP_WATERSHED_SEGMENT_BOUNDARIES_ONLY       /**< Image watershed segment boundary type render boundaries only. */
 } NppiWatershedSegmentBoundaryType;
 
@@ -999,6 +1001,17 @@ typedef struct
     Npp32f * pTwist;    /**< device memory pointer to the color twist matrix with floating-point coefficient values to be used for this image. */
 } NppiColorTwistBatchCXR;
 
+/** 
+ *  Color space enumeration
+ */
+typedef enum
+{
+    NPP_TRANSCODE_ONLY, /**< no color space defined. used for simple transcoding. */
+    NPP_BT601JPEG,      /**< BT601 colorspace used within JPEG images covering 0..255. */
+    NPP_BT709HDTV,      /**< BT709 High Definition TV colorspace. */
+    NPP_BT2020HDTV      /**< BT2020 High Definition TV colorspace. */
+} NppiColorSpace;
+
 /**
  * Profile data type for radial and linear profiles
  */
