@@ -148,6 +148,8 @@ typedef enum {
  *   Enable stack canaries in device code.
  *   Stack canaries make it more difficult to exploit certain types of memory safety bugs involving stack-local variables.
  *   The compiler uses heuristics to assess the risk of such a bug in each function.  Only those functions which are deemed high-risk make use of a stack canary.
+ * - \c -r \n
+ *   Do relocatable (or incremental) link, producing another relocatable object.
  */
 
 /**
@@ -163,7 +165,7 @@ typedef struct nvJitLink* nvJitLinkHandle; // opaque handle
 
 // For versioning we will have separate API version for each library version
 
-extern nvJitLinkResult __nvJitLinkCreate_13_0(
+extern nvJitLinkResult __nvJitLinkCreate_13_1(
   nvJitLinkHandle *handle,
   uint32_t numOptions,
   const char * const * options);
@@ -192,11 +194,16 @@ static inline nvJitLinkResult nvJitLinkCreate(
   uint32_t numOptions,
   const char **options)
 {
-  return __nvJitLinkCreate_13_0 (handle, numOptions, options);
+  return __nvJitLinkCreate_13_1 (handle, numOptions, options);
 }
+#else
+extern nvJitLinkResult nvJitLinkCreate(
+  nvJitLinkHandle *handle,
+  uint32_t numOptions,
+  const char **options);
 #endif
  
-extern nvJitLinkResult __nvJitLinkDestroy_13_0 (nvJitLinkHandle *handle);
+extern nvJitLinkResult __nvJitLinkDestroy_13_1 (nvJitLinkHandle *handle);
 /**
  * \ingroup linking
  * \brief   nvJitLinkDestroy frees the memory associated with the given handle
@@ -213,11 +220,13 @@ extern nvJitLinkResult __nvJitLinkDestroy_13_0 (nvJitLinkHandle *handle);
 #ifndef NVJITLINK_NO_INLINE
 static inline nvJitLinkResult nvJitLinkDestroy (nvJitLinkHandle *handle)
 {
-  return __nvJitLinkDestroy_13_0 (handle);
+  return __nvJitLinkDestroy_13_1 (handle);
 }
+#else
+extern nvJitLinkResult nvJitLinkDestroy (nvJitLinkHandle *handle);
 #endif
  
-extern nvJitLinkResult __nvJitLinkAddData_13_0(
+extern nvJitLinkResult __nvJitLinkAddData_13_1(
   nvJitLinkHandle handle,
   nvJitLinkInputType inputType,
   const void *data,
@@ -245,11 +254,18 @@ static inline nvJitLinkResult nvJitLinkAddData(
   size_t size,
   const char *name) // name can be null
 {
-  return __nvJitLinkAddData_13_0 (handle, inputType, data, size, name);
+  return __nvJitLinkAddData_13_1 (handle, inputType, data, size, name);
 }
+#else
+extern nvJitLinkResult nvJitLinkAddData(
+  nvJitLinkHandle handle,
+  nvJitLinkInputType inputType,
+  const void *data,
+  size_t size,
+  const char *name); // name can be null
 #endif
  
-extern nvJitLinkResult __nvJitLinkAddFile_13_0(
+extern nvJitLinkResult __nvJitLinkAddFile_13_1(
   nvJitLinkHandle handle,
   nvJitLinkInputType inputType,
   const char *fileName); // includes path to file
@@ -271,11 +287,16 @@ static inline nvJitLinkResult nvJitLinkAddFile(
   nvJitLinkInputType inputType,
   const char *fileName) // includes path to file
 {
-  return __nvJitLinkAddFile_13_0 (handle, inputType, fileName);
+  return __nvJitLinkAddFile_13_1 (handle, inputType, fileName);
 }
+#else
+extern nvJitLinkResult nvJitLinkAddFile(
+  nvJitLinkHandle handle,
+  nvJitLinkInputType inputType,
+  const char *fileName); // includes path to file
 #endif
  
-extern nvJitLinkResult __nvJitLinkComplete_13_0 (nvJitLinkHandle handle);
+extern nvJitLinkResult __nvJitLinkComplete_13_1 (nvJitLinkHandle handle);
 /**
  * \ingroup linking
  * \brief   nvJitLinkComplete does the actual link. 
@@ -289,11 +310,13 @@ extern nvJitLinkResult __nvJitLinkComplete_13_0 (nvJitLinkHandle handle);
 #ifndef NVJITLINK_NO_INLINE
 static inline nvJitLinkResult nvJitLinkComplete (nvJitLinkHandle handle)
 {
-  return __nvJitLinkComplete_13_0 (handle);
+  return __nvJitLinkComplete_13_1 (handle);
 }
+#else
+extern nvJitLinkResult nvJitLinkComplete (nvJitLinkHandle handle);
 #endif
  
-extern nvJitLinkResult __nvJitLinkGetLinkedCubinSize_13_0(
+extern nvJitLinkResult __nvJitLinkGetLinkedCubinSize_13_1(
   nvJitLinkHandle handle,
   size_t *size);
 /**
@@ -314,11 +337,15 @@ static inline nvJitLinkResult nvJitLinkGetLinkedCubinSize(
   nvJitLinkHandle handle,
   size_t *size)
 {
-  return __nvJitLinkGetLinkedCubinSize_13_0 (handle, size);
+  return __nvJitLinkGetLinkedCubinSize_13_1 (handle, size);
 }
+#else
+extern nvJitLinkResult nvJitLinkGetLinkedCubinSize(
+  nvJitLinkHandle handle,
+  size_t *size);
 #endif
 
-extern nvJitLinkResult __nvJitLinkGetLinkedCubin_13_0(
+extern nvJitLinkResult __nvJitLinkGetLinkedCubin_13_1(
   nvJitLinkHandle handle,
   void *cubin);
 /**
@@ -340,11 +367,15 @@ static inline nvJitLinkResult nvJitLinkGetLinkedCubin(
   nvJitLinkHandle handle,
   void *cubin)
 {
-  return __nvJitLinkGetLinkedCubin_13_0 (handle, cubin);
+  return __nvJitLinkGetLinkedCubin_13_1 (handle, cubin);
 }
+#else
+extern nvJitLinkResult nvJitLinkGetLinkedCubin(
+  nvJitLinkHandle handle,
+  void *cubin);
 #endif
  
-extern nvJitLinkResult __nvJitLinkGetLinkedPtxSize_13_0(
+extern nvJitLinkResult __nvJitLinkGetLinkedPtxSize_13_1(
   nvJitLinkHandle handle,
   size_t *size);
 /**
@@ -366,11 +397,15 @@ static inline nvJitLinkResult nvJitLinkGetLinkedPtxSize(
   nvJitLinkHandle handle,
   size_t *size)
 {
-  return __nvJitLinkGetLinkedPtxSize_13_0 (handle, size);
+  return __nvJitLinkGetLinkedPtxSize_13_1 (handle, size);
 }
+#else
+extern nvJitLinkResult nvJitLinkGetLinkedPtxSize(
+  nvJitLinkHandle handle,
+  size_t *size);
 #endif
 
-extern nvJitLinkResult __nvJitLinkGetLinkedPtx_13_0(
+extern nvJitLinkResult __nvJitLinkGetLinkedPtx_13_1(
   nvJitLinkHandle handle,
   char *ptx);
 /**
@@ -393,11 +428,15 @@ static inline nvJitLinkResult nvJitLinkGetLinkedPtx(
   nvJitLinkHandle handle,
   char *ptx)
 {
-  return __nvJitLinkGetLinkedPtx_13_0 (handle, ptx);
+  return __nvJitLinkGetLinkedPtx_13_1 (handle, ptx);
 }
+#else
+extern nvJitLinkResult nvJitLinkGetLinkedPtx(
+  nvJitLinkHandle handle,
+  char *ptx);
 #endif
  
-extern nvJitLinkResult __nvJitLinkGetErrorLogSize_13_0(
+extern nvJitLinkResult __nvJitLinkGetErrorLogSize_13_1(
   nvJitLinkHandle handle,
   size_t *size);
 /**
@@ -418,11 +457,15 @@ static inline nvJitLinkResult nvJitLinkGetErrorLogSize(
   nvJitLinkHandle handle,
   size_t *size)
 {
-  return __nvJitLinkGetErrorLogSize_13_0 (handle, size);
+  return __nvJitLinkGetErrorLogSize_13_1 (handle, size);
 }
+#else
+extern nvJitLinkResult nvJitLinkGetErrorLogSize(
+  nvJitLinkHandle handle,
+  size_t *size);
 #endif
 
-extern nvJitLinkResult __nvJitLinkGetErrorLog_13_0(
+extern nvJitLinkResult __nvJitLinkGetErrorLog_13_1(
   nvJitLinkHandle handle,
   char *log);
 /**
@@ -444,11 +487,15 @@ static inline nvJitLinkResult nvJitLinkGetErrorLog(
   nvJitLinkHandle handle,
   char *log)
 {
-  return __nvJitLinkGetErrorLog_13_0 (handle, log);
+  return __nvJitLinkGetErrorLog_13_1 (handle, log);
 }
+#else
+extern nvJitLinkResult nvJitLinkGetErrorLog(
+  nvJitLinkHandle handle,
+  char *log);
 #endif
 
-extern nvJitLinkResult __nvJitLinkGetInfoLogSize_13_0(
+extern nvJitLinkResult __nvJitLinkGetInfoLogSize_13_1(
   nvJitLinkHandle handle,
   size_t *size);
 /**
@@ -469,11 +516,15 @@ static inline nvJitLinkResult nvJitLinkGetInfoLogSize(
   nvJitLinkHandle handle,
   size_t *size)
 {
-  return __nvJitLinkGetInfoLogSize_13_0 (handle, size);
+  return __nvJitLinkGetInfoLogSize_13_1 (handle, size);
 }
+#else
+extern nvJitLinkResult nvJitLinkGetInfoLogSize(
+  nvJitLinkHandle handle,
+  size_t *size);
 #endif
 
-extern nvJitLinkResult __nvJitLinkGetInfoLog_13_0(
+extern nvJitLinkResult __nvJitLinkGetInfoLog_13_1(
   nvJitLinkHandle handle,
   char *log);
 /**
@@ -495,8 +546,12 @@ static inline nvJitLinkResult nvJitLinkGetInfoLog(
   nvJitLinkHandle handle,
   char *log)
 {
-  return __nvJitLinkGetInfoLog_13_0 (handle, log);
+  return __nvJitLinkGetInfoLog_13_1 (handle, log);
 }
+#else
+extern nvJitLinkResult nvJitLinkGetInfoLog(
+  nvJitLinkHandle handle,
+  char *log);
 #endif
 
 /**
