@@ -85,6 +85,10 @@ typedef enum {
 
     /* Indicates that compilation has been cancelled by the user */
     NVPTXCOMPILE_ERROR_CANCELLED                      = 9,
+
+    /* Indicates only parsing phase is completed successfully for the input PTX */
+    /* No cubin gets generated */
+    NVPTXCOMPILE_PARSE_ONLY_SUCCESS                   = 10,
 } nvPTXCompileResult;
 
 /* ----------------------------- PTX Compiler APIs ---------------------------- */
@@ -143,6 +147,34 @@ nvPTXCompileResult nvPTXCompilerGetNumSupportedVersions(int *numVersions);
  */
 nvPTXCompileResult nvPTXCompilerGetSupportedVersions(int (*versions)[2]);
 
+/** \ingroup versioning
+ *
+ * \brief            Sets the output paramater \p numArchs with the number of architectures supported by
+ *                   PTX Compiler Library.
+ *                   This can be used to pass an array to nvPTXCompilerGetSupportedArchs to get the supported architectures.
+ *
+ * \param            [out] numArchs   Number of supported architectures
+ *
+ * \return
+ *   - \link #nvPTXCompileResult NVPTXCOMPILE_SUCCESS \endlink
+ *   - \link #nvPTXCompileResult NVPTXCOMPILE_ERROR_INVALID_INPUT \endlink
+ */
+nvPTXCompileResult nvPTXCompilerGetNumSupportedArchs(int *numArchs);
+
+/** \ingroup versioning
+ *
+ * \brief            Populates the array passed via output paramater \p supportedArchs with architectures supported by
+ *                   PTX Compiler Library. The array is sorted in the ascending order.
+ *                   The size of the array to be passed can be determined using nvPTXCompilerGetNumSupportedArchs.
+ *
+ * \param            [out] supportedArchs   Sorted array of supported architectures.
+ *
+ * \return
+ *   - \link #nvPTXCompileResult NVPTXCOMPILE_SUCCESS \endlink
+ *   - \link #nvPTXCompileResult NVPTXCOMPILE_ERROR_INVALID_INPUT \endlink
+ */
+nvPTXCompileResult nvPTXCompilerGetSupportedArchs(int *supportedArchs);
+
 /**
  *
  * \defgroup compilation Compilation APIs
